/* #############################################################################
   *****************************************************************************
                  Copyright (c) 2010, Advantech eAutomation Division
                              Advantech Co., Ltd.
                               UNO Linux DIO driver
        This code is loosely based on uno2050 dio and unoisa dio drivers,written
         by Joshua.lan and others.
        THIS IS AN UNPUBLISHED WORK CONTAINING CONFIDENTIAL AND PROPRIETARY
           INFORMATION WHICH IS THE PROPERTY OF ADVANTECH AUTOMATION CORP.
  
      ANY DISCLOSURE, USE, OR REPRODUCTION, WITHOUT WRITTEN AUTHORIZATION FROM
                 ADVANTECH AUTOMATION CORP., IS STRICTLY PROHIBITED.
   *****************************************************************************
   #############################################################################
  
   File: 	    unodio.c
   Status: 	    works
  
   Permission to use, copy, modify, and distribute this software and its
   documentation for any purpose and without fee is hereby granted, provided
   that the above copyright notice appear in all copies and that both that
   copyright notice and this permission notice appear in supporting
   documentation.  This software is provided "as is" without express or
   implied warranty.
  ----------------------------------------------------------------------------*/

#include <linux/init.h>
#include <linux/module.h>
#include <linux/version.h>
#include <linux/types.h>
#include <linux/errno.h>
#include <linux/kernel.h>
#include <linux/sched.h>
#include <linux/interrupt.h>
#include <linux/ioport.h>
#include <linux/ioctl.h>
#include <asm/io.h>
#include <linux/pci.h>
#include <linux/poll.h>
#include <linux/fs.h>
#include <linux/param.h>
#include <asm/uaccess.h>
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 27)
#include <linux/kthread.h>
#endif

#include "uno_dio.h"
int g_con = 0;
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 27)
static struct task_struct * thread_pid[3] = { NULL, NULL, NULL};
#else
static int thread_pid[3] = { 0, 0, 0 };
#endif
static DECLARE_WAIT_QUEUE_HEAD(thread_wait);
static DECLARE_COMPLETION(thread_exited);
struct devinfo DevInfo;
static int CntStatus[3] = { 0, 0, 0 };
static int ClkcfgFlag = 0;

//#####################################################################
// *****************************************************************************
// Design Notes:  
// -----------------------------------------------------------------------------
int GetPowerStatus ( short* powerstatus )
{
	*powerstatus = inb(iobase + POWER_OFFSET);		
	DEBUGPRINT("Powerstatus is: %d\n", *powerstatus);
	return 0;
}
// *****************************************************************************
// Design Notes:  
// -----------------------------------------------------------------------------
int DioReadPortByte ( short port, short* value )
{
	DEBUGPRINT("[unodio - DioReadPortByte]: entry\r\n");
	if ( ( CntStatus[0] & (CNT_CLKEXT | CNT_GATEEXT) ) 
		|| (CntStatus[1] & (CNT_CLKEXT | CNT_GATEEXT) ) 
		|| ( CntStatus[2] & (CNT_CLKEXT | CNT_GATEEXT) ) )  
	{
		printk(KERN_ERR "unodio: DioReadPortByte Warning: Input pins are used by counter!\n");
	}
	*value = inb(iobase + DI_OFFSET);
	return 0;
}


// *****************************************************************************
// Design Notes:  
//-----------------------------------------------------------------------------
int DioWritePortByte (
	short port, 
	short mask, 
	short state )
{
	short ReadBack;
  
	DEBUGPRINT("[unodio - DioWritePortByte]: entry\r\n");
	if ( ( CntStatus[0] & CNT_OUTPUT ) 
		|| (CntStatus[1] & CNT_OUTPUT ) 
		|| ( CntStatus[2] & CNT_OUTPUT ) )
	{
		printk(KERN_ERR " DioWritePortByte Warning: Output pins are used by counter!\n");
	}
	state = state & 0xff;
	mask = mask & 0xff;    
  
	// read back the current state
	ReadBack = inb(iobase + DO_OFFSET);
   
	state = (state & mask) | (ReadBack & ~(mask));
 
	//  write the desired output state to the hardware 
	outb(state, iobase + DO_OFFSET);
  
	DEBUGPRINT("[unodio - DioWritePortByte]: leave\r\n");
  
	return 0;
}

// *****************************************************************************
// Design Notes:  
// -----------------------------------------------------------------------------
int DioReadBit (
	short port, 
	short bit, 
	short* state )
{
	short wMask;
  
	DEBUGPRINT("[unodio - DioReadBit]: entry\r\n");
  
	*state = inb(iobase + DI_OFFSET);

	if ( bit <= 7 ) 
	{
		wMask = 0x01;
		wMask = wMask << bit;
		*state = *state & wMask;
    
		if ( *state )
			*state = 1;        
		else
			*state = 0;     
	}
  
	DEBUGPRINT("[unodio - DioReadBit]: leave\r\n");
  
	return 0;
}


// *****************************************************************************
// Design Notes:  
// -----------------------------------------------------------------------------
int DioWriteBit (
	short port, 
	short bit, 
	short state )
{
	short mask;
	short ReadBack;
  
	DEBUGPRINT("[unodio - DioWriteBit]: entry\r\n");
	mask = 0x01;
	mask = mask << bit;
	state = state << bit;
  
	// read back the current state
	ReadBack = inb(iobase + DO_OFFSET);
	state = (state & mask) | (ReadBack & ~(mask));
  
	//  write the desired output state to the hardware 
	outb(state, iobase + DO_OFFSET);
  
	DEBUGPRINT("[unodio - DioWriteBit]: leave\r\n");
  
	return 0;
}

// *****************************************************************************
// Design Notes:  
// ----------------------------------------------------------------------------
int DioGetCurrentDOByte ( short port, short* value )
{
	int data;
	DEBUGPRINT("[unodio - DioGetCurrentDOByte]: entry\r\n");
	data = CNT_CLKEXT | CNT_GATEEXT;
	if ( ( CntStatus[0] & data ) 
		|| ( CntStatus[1] & data ) 
		|| ( CntStatus[2] & data ) )
	{
		printk( KERN_ERR " DioGetCurrentDOByte Warning: Input pins are used by counter!\n");
	}
	*value = inb(iobase + DO_OFFSET);
  
	return 0;
}

// *****************************************************************************
// Design Notes:  
// -----------------------------------------------------------------------------
int DioGetCurrentDOBit (
	short port, 
	short bit, 
	short* state )
{
	short mask;
	int ret;
  
	DEBUGPRINT("[unodio - DioGetCurrentDOBit]: entry\r\n");
  
	ret = inb(iobase + DO_OFFSET);
	*state = ret;
  
	if ( bit <= 7 )
 	{
		mask = 0x01;
		mask = mask << bit;
		*state = *state & mask;
    
		if (*state)
			*state = 1;        
		else
			*state = 0;            
	}
  
  	DEBUGPRINT("[unodio - DioGetCurrentDOBit]: leave\r\n");
  
	return 0;
}

// *****************************************************************************
// Design Notes:  
// -----------------------------------------------------------------------------
int CounterClockConfig ( short TimeBase )
{   	
	short ReadBack, data;   	    
  
	// ReadBack the current config value
	ReadBack = inb(iobase + TIMEBASE_OFFSET);
  
	data = (TimeBase & TIMEBASE_MASK) | (ReadBack & ~(TIMEBASE_MASK));
  
 	DEBUGPRINT("CounterConfig usData=0x%Xh\r\n", data); 
 	
  	// Liming added, 8/2/2005
	// test CounterClockConfig is busy or not
	if ( ( CntStatus[0] & CNT_CLKCFG ) 
		|| ( CntStatus[1] & CNT_CLKCFG ) 
		|| ( CntStatus[2] & CNT_CLKCFG ) )
	{
		if ( ReadBack == data )
		{
			return 0;
		}
		else
		{
			printk(KERN_ERR "Warning: Counter clock config has been changed!\n");
		}
	} 
	else
	{
		// set CounterClockConfig flag
		ClkcfgFlag = CNT_CLKCFG;
	}
	// Liming added end
	//  write the desired time base value to the hardware 
	outb(data, iobase + TIMEBASE_OFFSET);
  
	// save the config data to internal data pool
	DevInfo.TimeBase = TimeBase;
  
	return 0;    
}

// *****************************************************************************
// Design Notes:  
// -----------------------------------------------------------------------------
int CounterConfig (
	short counter, 
	short ClockPin, 
	short GatePin, 
	short OutPin )
{
	short status;
  	// Liming added for flags of counter, 8/2/2005
	if ( ( counter > COUNTER_32 ) || ( counter < COUNTER_0 ) ) 
	{
		return -1;
	}
	// test counter is busy or not
  	if ( CntStatus[ counter ] & CNT_BUSY )     
	{
		printk(KERN_ERR "Resource conflicts: Counter %d is busy now!\n", counter );
		return -1;
	}
	else
	{
		CntStatus[ counter ] |= CNT_BUSY;   // set busy flag
		if ( ClockPin )
		{
			CntStatus[ counter ] |= CNT_CLKEXT;
		}
		else
		{	
			CntStatus[ counter ] &= ~CNT_CLKEXT;
		}

		if ( GatePin )
		{
			CntStatus[ counter ] |= CNT_GATEEXT;
		}
		else
		{	
			CntStatus[ counter ] &= ~CNT_GATEEXT;
		}

		if ( OutPin )
		{	
			CntStatus[ counter ] |= CNT_OUTPUT;
		}
		else
		{	
			CntStatus[ counter ] &= ~CNT_OUTPUT;
		}

		if ( counter == 2)              // counter 2 is special
		{
			if ( ( CntStatus[0] & CNT_BUSY ) || ( CntStatus[1] & CNT_BUSY ))
			{
				CntStatus[2] = CNT_READY;
				printk(KERN_ERR "Resource conflicts: Counter 0 or 1 is busy now!\n");
				return -1;
			}
			else                   
			{
				CntStatus[0] = CntStatus[2]; // counter 0 busy
				CntStatus[1] = CntStatus[2]; // counter 1 busy 
			}
		}
	}
	// Liming add end
	// if counter channel is bigger than counter 1, that is cascade mode
	// that's config is the same with counter 1
	if (counter > COUNTER_1) 
		counter = COUNTER_1;  // 32-bit cascade mode
  
	status = inb(iobase + counter + CONT_SETTING_OFFSET); // read back
	DEBUGPRINT("CounterConfig:  ReadBack = %d\n", status);
  
	if (ClockPin == CFG_CLOCK_EXTERNAL)  // setting clock pin
		status |= CLOCK_EXTERNAL;
	else
		status &= CLOCK_INTERNAL;
  
	if (GatePin == CFG_GATE_EXTERNAL)  // setting gate pin
		status |= GATE_EXTERNAL;
	else
		status &= GATE_INTERNAL;
  
	if (OutPin == CFG_OUTPIN_TO_DO)   // setting output pin
		status |= OUTPUT_EXTERNAL;
	else
		status &= OUTPUT_INTERNAL;
 
	DEBUGPRINT("CounterConfig:  Write = %d\n", status);
   
	outb(status, iobase + counter + CONT_SETTING_OFFSET); // write done
      
	return 0; 
}

int CounterGateSet ( short counter, short CTRGate )
{
	short status;
  
	// if counter channel is bigger than counter 1, that is cascade mode
	// that's config is the same with counter 1
	if (counter > COUNTER_1) 
		counter = COUNTER_1;  // 32-bit cascade mode
  
	status = inb(iobase + counter + INTL_START_OFFSET); // read back
  
	if (CTRGate == CFG_CTR_START)   // setting ctr gate valu
		status |= CTR_COUNTER_START; 
	else
		status &= CTR_COUNTER_STOP; 
  
	outb(status, iobase + counter + INTL_START_OFFSET); // write done
	return 0; 
}

int CounterOutputMonitor ( short counter, short* state )
{
	short status;
  
	// if counter channel is bigger than counter 1, that is cascade mode
	// that's config is the same with counter 1
	if ( counter > COUNTER_1 ) 
		counter = COUNTER_1;  // 32-bit cascade mode    

	status = inb(iobase + counter + COUNTER_MONITOR_OFFSET); // read back      
	if (status & MONITOR_COUNTER)
		*state = 1;
	else
		*state = 0;
  
	return 0; 
}

int CounterStartIO ( 
	short counter, 
	short InitLoCount, 
	short InitHiCount )
{
	short ctr_byte;
	//set control byte for control register, use mode = 2;
	ctr_byte = counter << 6;
	//ctr_byte |= 0x30; // mode 0
	//  ctr_byte |= 0x32;   // mode 1
	ctr_byte |= 0x34;   // mode 2 :
	outb(ctr_byte, iobase + CTRL_OFFSET);
	outb(InitLoCount, iobase + (short)(counter+CONT_OFFSET));//write lowbyte
	outb(InitHiCount, iobase + (short)(counter+CONT_OFFSET));//write highbyte

	DEBUGPRINT("CounterStartIO ctr_byteG=0x%Xh\r\n", ctr_byte);
	DEBUGPRINT("CounterStartIO counter 0x%Xh\r\n", counter+CONT_OFFSET);
  
  return 0;
}

// *****************************************************************************
// Design Notes:  
// -----------------------------------------------------------------------------
int CounterTimerStart ( short counter, long InitHWCount )
{   
	short usCNT_A, usCNT_B, usReadBack, usData;
	short usCurrA_Low, usCurrA_High, usCurrB_Low, usCurrB_High;
  
	if ( counter > 2 )
		return -1;      
	// Liming added, 8/2/2005
	if ( CntStatus[ counter ] & CNT_BUSY )
	{
		printk(KERN_ERR "Resource conflicts: Counter %d is busy now!\n", counter );

		return -1;
	}
	else 
	{
		// set busy flag for counter
		CntStatus[ counter ] |= CNT_BUSY | ClkcfgFlag;
		if ( counter == 2 )
		{
			if ( ( CntStatus[0] & CNT_BUSY )
				|| ( CntStatus[1] & CNT_BUSY ) )
			{
				CntStatus[2] = CNT_READY;
				printk(KERN_ERR "Resource conflicts: Counter 0 or 1 is busy!\n");
				return -1;
			}
			else
			{
				CntStatus[0] = CntStatus[2];
				CntStatus[1] = CntStatus[2];
			}
		}
	}		
	// Liming add ended               
	// read back some data
	usCNT_A = inb(iobase + CONT_A_OFFSET);
	usCNT_B = inb(iobase + CONT_B_OFFSET);
  
	usCurrA_Low  = (short)(InitHWCount & 0xff);
	usCurrA_High = (short)((InitHWCount >> 8) & 0xff);
	usCurrB_Low  = (short)((InitHWCount >> 16) & 0xff);
	usCurrB_High = (short)((InitHWCount >> 24) & 0xff);
  
	// 1. setting cascade mode or not    
	// 2. setting the DI_CLKA or DI_CLKB
	// 3. setting the GATEA or GATEB
	switch (counter) 
	{
	case COUNTER_0:  // 16-bit counter A
	case COUNTER_1:  // 16-bit counter B            
		usCNT_B = usCNT_B & CASCADE_OFF; // set cascade to off (0)                   
		if (counter == COUNTER_0)
			usCNT_A = usCNT_A & EXTCLK_OFF;
		else
			usCNT_B = usCNT_B & EXTCLK_OFF;            
      
		outb(usCNT_A, iobase + CONT_A_OFFSET);
		outb(usCNT_B, iobase + CONT_B_OFFSET);                        	              
		CounterStartIO(counter, usCurrA_Low, usCurrA_High);			      
		// enable one pulse to reload the count value for 8254 bug
		// for timer mode, its GateMode is always is 1
		usReadBack = inb(iobase + INTL_START_OFFSET + counter);
		usData = usReadBack | START_COUNTER | ONE_PULSE_COUNTER; 
		outb(usData, iobase + INTL_START_OFFSET + counter);
	break;        
     
	case COUNTER_32: // 32-bit counter A+B, cascade mode      
		usCNT_B = usCNT_B | CASCADE_ON;  // set cascade to on (1)
		usCNT_B = usCNT_B & EXTCLK_OFF;            
		outb(usCNT_B, iobase + CONT_B_OFFSET);            
      
		CounterStartIO(COUNTER_0, usCurrA_Low, usCurrA_High);          
  		CounterStartIO(COUNTER_1, usCurrB_Low, usCurrB_High);          
		// enable one pulse to reload the count value for 8254 bug
		// for timer mode, its GateMode is always is 1        	
		usReadBack = inb(iobase + INTL_START_OFFSET+COUNTER_1);
		usData = usReadBack | START_COUNTER | ONE_PULSE_COUNTER;
		outb(usData, iobase + INTL_START_OFFSET+COUNTER_1);
		break;
	}
  
	return 0;
}

// Liming masked for no use, 06/10/2005
/*
// *****************************************************************************
// Design Notes:  
// -----------------------------------------------------------------------------
int CounterReadIO32 ( short* CurrA, short* CurrB )
{
	short ctr_byte, low_byte, high_byte;
	int  lCurrent_Count;
  
	// Read first counter
	ctr_byte = 0xd6; // to latch CNT0 and CNT1
	outb(ctr_byte, iobase + CTRL_OFFSET);
  
	low_byte =  inb(iobase + (short)(COUNTER_0+CONT_OFFSET));//read low byte
	high_byte = inb(iobase + (short)(COUNTER_0+CONT_OFFSET));//read highbyte 	
	*CurrA = ((high_byte << 8) & 0xff00) | (low_byte & 0x00ff);
  
	DEBUGPRINT("CounterReadIO Counter A ==> low byte=%x\r\n", low_byte); 
	DEBUGPRINT("CounterReadIO Counter A ==>high byte=%x\r\n", high_byte); 
	DEBUGPRINT("CounterReadIO Counter A ==>current_count=%d hex=%x\r\n", *CurrA, *CurrA); 		
  
	low_byte =  inb(iobase + (short)(COUNTER_1+CONT_OFFSET)); //read lowbyte
	high_byte = inb(iobase + (short)(COUNTER_1+CONT_OFFSET)); //readhighbyte

	*CurrB = ((high_byte << 8) & 0xff00) | (low_byte & 0x00ff);    
  
	DEBUGPRINT("CounterReadIO Counter B ==> low byte=%x\r\n", low_byte); 
	DEBUGPRINT("CounterReadIO Counter B ==>high byte=%x\r\n", high_byte); 
	DEBUGPRINT("CounterReadIO Counter B ==>current_count=%d hex=%x\r\n", *CurrB, *CurrB); 	    
  
	lCurrent_Count  = (*CurrB) << 16; // shift low word
	lCurrent_Count |= (*CurrA);       // combine high word
  
	return lCurrent_Count;    
}
*/
// Liming masked ended

// *****************************************************************************
// Design Notes:  
// -----------------------------------------------------------------------------
unsigned short CounterReadIO ( short counter )
{
	short ctr_byte, low_byte, high_byte, current_count;        
  
	// Read first counter
	ctr_byte = counter << 6;
	ctr_byte |= 0x00; // mode 0
	//ctr_byte |= 0x04; 	// mode 2
	outb(ctr_byte, iobase + CTRL_OFFSET);
  
	low_byte =  inb(iobase + (short)(counter+CONT_OFFSET)); //read low byte
	high_byte = inb(iobase + (short)(counter+CONT_OFFSET)); //read high byte
  
	DEBUGPRINT("CounterReadIO counter=%x\r\n", counter); 
	DEBUGPRINT("CounterReadIO  low byte=%x\r\n", low_byte);
	DEBUGPRINT("CounterReadIO high byte=%x\r\n", high_byte); 
  
	current_count = ((high_byte << 8) & 0xff00) | (low_byte & 0x00ff);
  
	DEBUGPRINT("CounterReadIO current_count=%d hex=%x\r\n", current_count, current_count); 
  
	return current_count;    
}

// *****************************************************************************
// Design Notes:  
// CounterEventStart
// GateMode : 0  internal control start/stop
// GeteMode : 1  external control start/stop 
//------------------------------------------------------------------------------
int CounterEventStart ( short counter, short GateMode )
{
	short usCNT_A, usCNT_B;
	// short usCurr_A, usCurr_B;
	short usReadBack, usData;
  
	if (counter > 2)
		return -1;
 	CntStatus[ counter ] |= ClkcfgFlag; 
	// read back some data
	usCNT_A = inb(iobase + CONT_A_OFFSET);
 	usCNT_B = inb(iobase + CONT_B_OFFSET);
  
 	DEBUGPRINT(">>CounterEventStart CNTA_REG=0x%Xh\r\n", usCNT_A);
	DEBUGPRINT(">>CounterEventStart CNTB_REG=0x%Xh\r\n", usCNT_B);
	DEBUGPRINT(">>CounterEventStart counter=%d\r\n", counter);
  
	// 1. setting cascade mode or not    
	// 2. setting the DI_CLKA or DI_CLKB
	// 3. setting the GATEA or GATEB
	switch (counter)
	{
	case COUNTER_0:  // 16-bit counter A
	case COUNTER_1:  // 16-bit counter B
		//initialize some variable
 		DevInfo.OverFlow[counter]  = 0;
		DevInfo.FirstCount[counter] = 1;
		usCNT_B = usCNT_B & CASCADE_OFF; // 1
		outb(usCNT_B, iobase + CONT_B_OFFSET);
		CounterStartIO(counter, INIT_0XFF, INIT_0XFF);
    
		// enable one pulse to reload the count value for 8254 bug
		usReadBack = inb(iobase + INTL_START_OFFSET+counter);
	if (GateMode)
		usData = usReadBack | START_COUNTER | ONE_PULSE_COUNTER;
	else
		// Joshua modified 03/10/2005
		//usData = usReadBack | ONE_PULSE_COUNTER;
		usData = usReadBack & 0xfe;
		outb(usData, iobase + INTL_START_OFFSET+counter);     
		DevInfo.PreCount[counter] = CounterReadIO(counter);
	break;
    
	case COUNTER_32: // 32-bit counter A+B, cascade mode        
		// initialize some variable                    
		DevInfo.OverFlow32 = 0;	 
		DevInfo.FirstCount32 = 1;    
		usCNT_B = usCNT_B | CASCADE_ON;  // 1            
		outb(usCNT_B, iobase + CONT_B_OFFSET);    
		CounterStartIO(COUNTER_0, INIT_0XFF, INIT_0XFF);
              	CounterStartIO(COUNTER_1, INIT_0XFF, INIT_0XFF);
     
		// enable one pulse to reload the count value for 8254 bug
		usReadBack = inb(iobase + INTL_START_OFFSET+COUNTER_1);
		if (GateMode)
			usData = usReadBack | START_COUNTER | ONE_PULSE_COUNTER;
		else
			// Joshua modified 03/10/2005
			//usData = usReadBack | ONE_PULSE_COUNTER;
			usData = usReadBack & 0xfe;
		outb(usData, iobase + INTL_START_OFFSET+COUNTER_1);
     		/* 
		CounterReadIO32(&usCurr_A, &usCurr_B);
		DevInfo.PreCountLow  = usCurr_A;
		DevInfo.PreCountHigh = usCurr_B;*/            
		DevInfo.PreCounter32 = 0;
	break;
	}
  
	return 0;
}

// *****************************************************************************
// Design Notes:  
// -----------------------------------------------------------------------------
int CounterEventRead32 (
	short* overflow, 
	unsigned long* count )
{       
	//short usCurr_A, usCurr_B;    
	unsigned long  c_data; 
	unsigned short c_data_A, c_data_B;// carray;
   
	// Liming modified for get value counter 0 and 1, 06/10/2005
	/* 
	CounterReadIO32(&usCurr_A, &usCurr_B);
	// get counter A value
	c_data_A = ((unsigned long)65535L - (unsigned long)usCurr_A); // convert to up count 
  
	// get counter B value
	c_data_B = ((unsigned long)65535L - (unsigned long)usCurr_B); // convert to up count 
  	*/
	// wait for first count
	if ( DevInfo.FirstCount32 )
	{
		*count = 0L;
		DevInfo.FirstCount32 = 0;
		return 0;
	}
	c_data_A = ( unsigned short )65535 - ( unsigned short )CounterReadIO( COUNTER_0 );
	c_data_B = ( unsigned short )65535 - ( unsigned short )CounterReadIO( COUNTER_1 );
	//Liming modification ended

	c_data  = c_data_B << 16; // shift low word
	c_data |= c_data_A;       // combine high word    
	//c_data -= c_data_B;       // the real data is only 65535
  
	// check if 16-bit counter register of 8254 is over flow   
	if(DevInfo.PreCounter32 > c_data)  // over flow    
		DevInfo.OverFlow32 += 1;   // increament overflow by 1          
  
	// Don't calculate the overflow and combine the real data
	// the output is only unsigned long, so no refer to overflow
  
	// check 32bit overflow ?, first clear overflow flag
	*overflow = 0;
	if (DevInfo.OverFlow32)
		*overflow = 1;    
  
	*count = c_data;                   // return data
	DevInfo.PreCounter32 = c_data;  // save data
	return 0; 
}

// *****************************************************************************
// Design Notes:  
// -----------------------------------------------------------------------------
int CounterEventRead ( 
	short counter, 
	short* overflow, 
	unsigned long * count )
{
	unsigned short current_count;
	int  c_data, carry;
  
	if (counter > 2)
		return -1;                
  
	if (counter == COUNTER_32)
		return CounterEventRead32(overflow, count);    
  
	current_count = CounterReadIO(counter);
  
	// Liming added 06/10/2005
	// wait for first valid count
	if ( DevInfo.FirstCount[ counter ] )
	{
		if ( DevInfo.PreCount[ counter ] == current_count )
		{
			*count = 0L;
			return 0;
		}
		else
		{
			DevInfo.FirstCount[ counter ] = 0;
			DevInfo.PreCount[ counter ] = 65535;
		}
	}
	// Liming add ended

	c_data = ((unsigned long)65535 - (unsigned long)current_count); // convert to up count 
  
	/* check if 16-bit counter register of 8254 is over flow     */
	if( ( DevInfo.PreCount[ counter ] < current_count ) && ( 0 == DevInfo.FirstCount[ counter ] ) )  // over flow
	{
		DevInfo.OverFlow[counter] += 1;   // increament overflow by 1 
  	}

	carry = DevInfo.OverFlow[counter] << 16; // shift overflow 16 bits 
	c_data |= carry;        // convert 16-bit to 32-bit 
	//c_data -= DevInfo.OverFlow[counter];     // the real data is only 65535
  
	// check 32bit overflow ?
	*overflow = 0;  		// clear overflow flag
	if (DevInfo.OverFlow[counter] > 65535L)
		*overflow = 1;
  
	*count = c_data;    // return data    
	DevInfo.PreCount[counter] = current_count;   // save data
  
	return 0; 
}

// Joshua add 03/10/2005
int freq_thread( void * param )
{
        unsigned short counter, cur_count, up_count, high, low;
        unsigned short /*regdata,*/ iobase, reg_rb;
        unsigned long cur_time;
        struct devinfo *DevInfo;

        DevInfo = param;
        counter = DevInfo->counter;
        iobase  = DevInfo->iobase;
        DEBUGPRINT("-->fthread!\n");
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 27)
	allow_signal(SIGTERM);
#endif
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 27)
        while ( DevInfo->freq_thread_flag[counter] == 1)
#else
        //while ( DevInfo->freq_thread_flag[counter] )
        for ( ; ; )
#endif
        {
                current->state = TASK_INTERRUPTIBLE;
                if (signal_pending(current))
                        break;

                if (counter == COUNTER_0 || counter == COUNTER_1 )
                {
                        //schedule_timeout( 102 );
                        schedule_timeout( HZ / 10 );
			if ( counter == COUNTER_0 )
                                outb( 0x00, iobase+ CTRL_OFFSET );
			else
				outb( 0x40, iobase + CTRL_OFFSET );
                        low = inb(iobase+counter + CONT_OFFSET);
                        high= inb( iobase + counter + CONT_OFFSET );
                        cur_count = ((high<<8)&0xff00)|(low&0x00ff);
                        cur_time = jiffies;
                        up_count = 65535 - cur_count;
                        if ( up_count > 63000 )
                        {
                                switch( counter )
                                {
                                case COUNTER_0:
                                        outb( 0x30, iobase+CTRL_OFFSET );
                                break;
                                case COUNTER_1:
                                        outb( 0x70, iobase+CTRL_OFFSET );
                                break;
                                }
                                outb(0xff, iobase + counter + CONT_OFFSET);
                                outb(0xff, iobase + counter + CONT_OFFSET);
                                outb(0x1, iobase+counter+INTL_START_OFFSET);
                                reg_rb = inb( iobase+counter+INTL_START_OFFSET);                                outb(reg_rb|START_COUNTER|ONE_PULSE_COUNTER,
                                        iobase + counter + INTL_START_OFFSET );
                                DevInfo->FreqPreReTicks[counter] =
                                        DevInfo->FreqCurReTicks[counter];
				DevInfo->FreqCurReTicks[counter] = cur_time;
                                //DevInfo->FreqPreReData[counter] = DevInfo->FreqCurReData[counter];
                        }
                        DevInfo->FreqPreData[counter] = DevInfo->FreqCurData[counter];
                        DevInfo->FreqPreTicks[counter]= DevInfo->FreqCurTicks[counter];
                        DevInfo->FreqCurData[counter] = up_count;
                        DevInfo->FreqCurTicks[counter]= cur_time;
                }
        }
        complete_and_exit( &thread_exited, 0 );
        //return 0;
}
// add end

// Design Notes:  
// -----------------------------------------------------------------------------
int CounterFreqStart ( 
	short counter, 
	short GatePeriod,
	short GateMode )
{
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 27)
	struct task_struct * pid;
#else
	int pid;
#endif
	short usReadBack;
	short usCNT_A, usCNT_B, usData;
  
	if (counter > COUNTER_1)
		return -1;                        

        // Joshua add 03/10/05 to reset the counter
        DevInfo.counter = counter;
        DevInfo.iobase  = iobase;
        switch( counter )
        {
        case COUNTER_0:
                outb( 0x30, iobase + CTRL_OFFSET        ); // mode 0
                CounterStartIO(counter, INIT_0XFF, INIT_0XFF);
                outb( 0x1, iobase + INTL_START_OFFSET + counter );
                break;
        case COUNTER_1:
                outb( 0x70, iobase + CTRL_OFFSET ); // mode 0
                CounterStartIO( counter, INIT_0XFF, INIT_0XFF );
                outb( 0x1, iobase + INTL_START_OFFSET + counter );
                break;
        }
        // add end

 	DevInfo.FreqCurTicks[counter] = jiffies;
	DevInfo.FreqCurData[counter] = 0; 
	// read back some data
	usCNT_A = inb(iobase + CONT_A_OFFSET);
	usCNT_B = inb(iobase + CONT_B_OFFSET);
  
	DEBUGPRINT(">>CounterEventStart CNTB_REG=0x%Xh\r\n", usCNT_B);         
	// 1. setting cascade mode or not    
	// 2. setting the DI_CLKA or DI_CLKB
	// 3. setting the GATEA or GATEB    
	// GateMode configure    
	switch (counter)
	{
	case COUNTER_0:  // 16-bit counter A        
		usCNT_B = usCNT_B & CASCADE_OFF; // 1
		outb(usCNT_A, iobase + CONT_A_OFFSET);
		outb(usCNT_B, iobase + CONT_B_OFFSET);
		CounterStartIO(counter, INIT_0XFF, INIT_0XFF);            
            
		// enable one pulse to reload the count value for 8254 bug
		usReadBack = inb(iobase + INTL_START_OFFSET+counter);
		if (GateMode)
			usData = usReadBack | START_COUNTER | ONE_PULSE_COUNTER;
		else
			usData = usReadBack & 0xfe;
			outb(usData, iobase + INTL_START_OFFSET+counter);
	break;
      
	case COUNTER_1:  // 16-bit counter B             
		usCNT_B = usCNT_B & CASCADE_OFF; // 1      
		outb(usCNT_A, iobase + CONT_A_OFFSET);
		outb(usCNT_B, iobase + CONT_B_OFFSET);                        	
		CounterStartIO(counter, INIT_0XFF, INIT_0XFF);            
      
		// enable one pulse to reload the count value for 8254 bug
		usReadBack = inb(iobase + INTL_START_OFFSET+COUNTER_1);
		if (GateMode)
			usData = usReadBack | START_COUNTER | ONE_PULSE_COUNTER;
		else
			//usData = usReadBack | ONE_PULSE_COUNTER;
			usData = usReadBack & 0xfe;
		outb(usData, iobase + INTL_START_OFFSET+COUNTER_1);
	break;            
      
	default:
		return -1;
	}        
  
	/* initial static variables for event counter */
        DevInfo.FreqPreReTicks[counter] = DevInfo.FreqCurTicks[counter];
        DevInfo.FreqCurReTicks[counter] = DevInfo.FreqCurTicks[counter];
        DevInfo.FreqPreTicks[counter] = DevInfo.FreqCurTicks[counter];
        DevInfo.FreqPreData[counter]  = 0;
        /* set control byte for  control register */
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 27)
	DevInfo.freq_thread_flag[counter] = 1;
        pid =  kthread_create(freq_thread,&DevInfo,"freq_thread");
#else
        pid = kernel_thread(freq_thread, &DevInfo, CLONE_FS | CLONE_FILES );
#endif
        if (!pid)
        {
                printk("Failed to start kernel_thread!\n");
                return -1;
        }
        thread_pid[counter] = pid;
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 27)
	wake_up_process(pid);
#endif

        return 0;
}

// Joshua masked and modified this function @03/10/2005
int CounterFreqRead (
        short counter,
        unsigned long *timelapse,
        long *cntnum,
	long *kernelhz )
{
	static long oldtimelapse = 0, oldcntnum = 0 ;
	long temptime = 0, tempcnt = 0;
	*kernelhz = HZ;
        if (( counter < 0)||(counter > 2))
                return -1;
        if (counter == COUNTER_32 )
                counter = COUNTER_1;
        // current value < previous one, so the counter must be reset
        if ( DevInfo.FreqPreData[counter]
                > DevInfo.FreqCurData[counter])
        {
                DEBUGPRINT("Counter Reset!\n");
                // ticktime overflow
                if ( DevInfo.FreqCurTicks[counter]
                        < DevInfo.FreqPreTicks[counter] )
                {
                        if ( DevInfo.FreqCurTicks[counter]
                                == DevInfo.FreqPreReTicks[counter])
                        {
                                DEBUGPRINT("1 CFR: ft is running!\n");
                                *cntnum = 0;
                                return 0;
                        }
                        //we usually do not need to care the jiffies overflow
                }
                else if ( DevInfo.FreqCurTicks[counter]
                        == DevInfo.FreqPreTicks[counter] )
                {
                        DEBUGPRINT("2 CFR: CurT=PreT\n");
                        *cntnum = 0;
                        return 0;
                }
                else
                {
                        if ( DevInfo.FreqCurTicks[counter]
                                == DevInfo.FreqPreReTicks[counter])
                        {
                                DEBUGPRINT("3 CFR: ");
                                *cntnum = 0;
                                return 0;
                        }
                        else
                        {
                                *timelapse = DevInfo.FreqCurTicks[counter]
                                        - DevInfo.FreqPreReTicks[counter];
                        }
                }
                *cntnum = DevInfo.FreqPreData[counter]+DevInfo.FreqCurData[counter];
        }
        else
        {
                DEBUGPRINT("CFR:counter doesn't reset!\n");
                // time tick overflow
                if ( DevInfo.FreqCurTicks[counter]
                        < DevInfo.FreqPreTicks[counter])
                {
                        if ( DevInfo.FreqCurTicks[counter]
                                == DevInfo.FreqCurReTicks[counter])
                                *timelapse = DevInfo.FreqCurTicks[counter]
                                        - DevInfo.FreqPreReTicks[counter];
                }
                else if ( DevInfo.FreqCurTicks[counter]
                        == DevInfo.FreqPreTicks[counter])
                {
                        DEBUGPRINT("4 CFR: CurT=PreT!\n");
                        *cntnum = 0;
                        return 0;
                }
                else
                {
                        if ( DevInfo.FreqCurTicks[counter]
                                ==DevInfo.FreqCurReTicks[counter] )
                                *timelapse = DevInfo.FreqCurTicks[counter]
                                        - DevInfo.FreqPreReTicks[counter];
                        else
                                *timelapse = DevInfo.FreqCurTicks[counter]
                                        - DevInfo.FreqCurReTicks[counter];

                }
                *cntnum = DevInfo.FreqCurData[counter];
		//lipeng add at 2006/07/31 to fixed the frequency calculation erro
		//static long oldtimelapse = 0, oldcntnum = 0 ;
		//long temptime = 0, tempcnt = 0;
DEBUGPRINT("4 CFR: timelapse %ld, cntnum %ld,oldtimelapse %ld, oldcntnum %ld\n", *timelapse, *cntnum,oldtimelapse,oldcntnum);


		temptime     = oldtimelapse;
		tempcnt      = oldcntnum;
		oldtimelapse = *timelapse;
		oldcntnum    = *cntnum;
		if ( *timelapse > temptime )
			*timelapse -= temptime;
		if ( *cntnum >= tempcnt )
			*cntnum -= tempcnt; 
                //lipeng add end
        }
        return 0;
}
/*
// *****************************************************************************
// Design Notes:  
// -----------------------------------------------------------------------------
int CounterFreqRead (
	short counter,
	unsigned long  *timelapse,
	long  *cntnum )  
{
	unsigned short  current_count;
	unsigned long  c_data, carry, dwTicks;
	//int  dwTicks;
	//double  fSampleRate;
  
	if (counter > COUNTER_1)
		return -1;
  
	current_count = CounterReadIO(counter);
	dwTicks       = jiffies;

	// still wait for first valid count
	c_data = ((unsigned long)65535L - (unsigned long)current_count); // convert to up count 

	// check if 16-bit counter register of 8253 is over flow ?    
	if(DevInfo.FreqPreCount[counter] < current_count)  // over flow
		DevInfo.FreqOverFlow[counter] += 1;  // increament overflow by 1 

	carry = DevInfo.FreqOverFlow[counter] << 16; // shift overflow 16 bits 
	c_data |= carry;        // convert 16-bit to 32-bit
	c_data -= DevInfo.OverFlow[counter];     // the real data is only 65535 every cycle

	if (DevInfo.FreqPreTicks[counter] == dwTicks)    // system timer check
		return -1;
	// Joshua modified
	*timelapse = ( dwTicks - DevInfo.FreqPreTicks[ counter ])* 10;  // convert the unit to 1ms
	*cntnum    = c_data - DevInfo.FreqPreData[ counter ];
	//fSampleRate = 100.0 / (double)(dwTicks - DevInfo.FreqPreTicks[counter]);  // jiffies is 10ms round up
	// *freq = (float)(c_data - DevInfo.FreqPreData[counter]) * (float)fSampleRate;
	// save data
	DevInfo.FreqPreCount[counter] = current_count;
	DevInfo.FreqPreData[counter]  = c_data;
	DevInfo.FreqPreTicks[counter] = dwTicks;

	return 0;
}
*/
// *****************************************************************************
// Design Notes:  
// -----------------------------------------------------------------------------
int CounterPulseStart (
	short counter, 
	long  period, 
	long  UpCycle, 
	short GateMode )   
{
	unsigned long divisor = 0;
	short usReadBack, usData;
	short ctr_byte, low_byte, high_byte;

	if ( counter > COUNTER_1 )
		return -1;
	CntStatus[ counter ] |= ClkcfgFlag;
	// configure counter 0 for internal clocking, by usTimeBase  
	switch ( DevInfo.TimeBase )
	{
	case TIME_BASE_100K:
		divisor = period;
	break;
	case TIME_BASE_10K:
		divisor = (int)(period/10);
	break;
	case TIME_BASE_1K:
		divisor = (int)(period/100);
	break;
	case TIME_BASE_100:
		divisor = (int)(period/1000);
	break;
	}
	// enable one pulse to reload the count value for 8254 bug
	usReadBack = inb(iobase + INTL_START_OFFSET + counter);
	if (GateMode)
		usData = usReadBack | START_COUNTER | ONE_PULSE_COUNTER;
	else
		// Joshua modified 03/10/2005	
		usData = usReadBack & 0xfe;	
		//usData = usReadBack | ONE_PULSE_COUNTER;
		outb(usData, iobase + INTL_START_OFFSET + counter);
 
	ctr_byte = counter << 6;
	ctr_byte |= 0x36;
	outb(ctr_byte, iobase + CTRL_OFFSET);

	low_byte = (unsigned short)((divisor/1000) & 0x000000FF);
	high_byte = (unsigned short)(((divisor/1000) &  0x0000FF00) >> 8);
	outb(low_byte, iobase + (counter+CONT_OFFSET));  /* write low byte */
	outb(high_byte, iobase + (counter+CONT_OFFSET));  /* write high byte */

	return 0;
}

int CounterReset ( short counter )
{
	short ReadBack, data;
	DEBUGPRINT(">>CounterReset channel=0x%Xh\r\n", counter);         
	if ( counter > 2 )
		return -1;
                
	DEBUGPRINT(">>CounterReset channel=0x%Xh\r\n", counter);        
	// Liming added for reset flags of counter, 8/3/2005
	// test counter is busy or not, if busy, leave it alone, and if not, 
	// restore it to 0.
	if ( !( CntStatus[ counter ] & CNT_BUSY ) )
		CntStatus[ counter ] &= CNT_READY;  


	// set selected counter ready 
	if (  counter == 2 )         // counter 2 is special
	{
		CntStatus[ 0 ] = CNT_READY;
		CntStatus[ 1 ] = CNT_READY;
		CntStatus[ 2 ] = CNT_READY;
	}
	else
	{
		CntStatus[ counter ] = CNT_READY;
	}
	if ( thread_pid[counter] )
	{

#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 27)
	        //kill_pid(thread_pid[counter], SIGTERM, 1);
		DevInfo.freq_thread_flag[counter] = 0;
		send_sig(SIGTERM,thread_pid[counter],1);
#else
	        kill_proc(thread_pid[counter], SIGTERM, 1);
#endif
        	wait_for_completion(&thread_exited);
		thread_pid[counter] = 0;
	}
	// Liming added ended
	         
	switch ( counter )
	{
	case COUNTER_0:
		outb(0x0, iobase + CONT_A_OFFSET);
		outb(0x30, iobase + CTRL_OFFSET);
		outb(0xff, iobase + (short)(COUNTER_0+CONT_OFFSET));  /* write low byte */
		outb(0xff, iobase + (short)(COUNTER_0+CONT_OFFSET));  /* write high byte */            
		outb(0x0, iobase + INTL_START_OFFSET+counter);  
	break;
            
	case COUNTER_1:
		// Liming added, 8/5/2005
		ReadBack = inb(iobase + CONT_B_OFFSET);
		data = ReadBack & 0x30;
		// Liming add ended
		outb(data, iobase + CONT_B_OFFSET);
		outb(0x70, iobase + CTRL_OFFSET); // channel B
		outb(0xff, iobase + (short)(COUNTER_1+CONT_OFFSET));  /* write low byte */
		outb(0xff, iobase + (short)(COUNTER_1+CONT_OFFSET));  /* write high byte */            
		outb(0x0, iobase + INTL_START_OFFSET+counter); 
	break;
      
	case COUNTER_32:
		// COUNTER_0
		outb(0x0, iobase + CONT_A_OFFSET);  
		outb(0x30, iobase + CTRL_OFFSET);
		outb(0xff, iobase + (short)(COUNTER_0+CONT_OFFSET));  /* write low byte */
		outb(0xff, iobase + (short)(COUNTER_0+CONT_OFFSET));  /* write high byte */                        
		outb(0x0, iobase + INTL_START_OFFSET+COUNTER_0);  
                        
		// COUNTER_1
		outb(0x0, iobase + CONT_B_OFFSET);
		outb(0x70, iobase + CTRL_OFFSET);
		outb(0xff, iobase + (short)(COUNTER_1+CONT_OFFSET));  /* write low byte */
		outb(0xff, iobase + (short)(COUNTER_1+CONT_OFFSET));  /* write high byte */            
		outb(0x0, iobase + INTL_START_OFFSET+COUNTER_1);
	break;     
      
	default:
		return -1;        
	}

	return 0; 
}

// *****************************************************************************
// Design Notes:  
// -----------------------------------------------------------------------------
int EventEnable (
	short EventType, 
	short Enabled, 
	short Count )
{
	unsigned short usStatusDI, usStatusCNTA, usStatusCNTB;
  
	DEBUGPRINT("->EventEnable\n");
    
	// read back some hardware values
	usStatusDI   = inb(iobase + DI_INT_OFFSET);
	usStatusCNTA = inb(iobase + CONT_A_OFFSET);
	usStatusCNTB = inb(iobase + CONT_B_OFFSET);       
  
	if (Enabled) 
	{       
		//printk("EventType = 0x%x, maks=0x%x\n",EventType, EVENT_DI0_MASK);    
		if (EventType & EVENT_DI0_MASK)                
			usStatusDI |= INT_DI0_EN;
    
		if (EventType & EVENT_DI1_MASK)        
			usStatusDI |= INT_DI1_EN;                        
    
		outb(usStatusDI, iobase + DI_INT_OFFSET);
    
		if (EventType & EVENT_CONTA_MASK)
			outb((usStatusCNTA | INT_CNT_EN), iobase + CONT_A_OFFSET);  
    
		if (EventType & EVENT_CONTB_MASK)
			outb((usStatusCNTB | INT_CNT_EN), iobase + CONT_B_OFFSET);
	}
	else 
	{
		if (EventType & EVENT_DI0_MASK)
			usStatusDI &= INT_DI0_DIS;
		if (EventType & EVENT_DI1_MASK)
			usStatusDI &= INT_DI1_DIS;
   
		outb(usStatusDI, iobase + DI_INT_OFFSET);
    
		if (EventType & EVENT_CONTA_MASK)
			outb((usStatusCNTA & INT_CNT_DIS), iobase + CONT_A_OFFSET);  
		if (EventType & EVENT_CONTB_MASK)
			outb((usStatusCNTB & INT_CNT_DIS), iobase + CONT_B_OFFSET);
	}            
  
	return 0; 
}

// *****************************************************************************
// Design Notes:  
// -----------------------------------------------------------------------------
int EventConfig ( short EventType, short RisingEdge )
{
	short usStatus;
	DEBUGPRINT("->EventConfig\n");
	// just support DI0 and DI1 event types
	// Liming added, 8/5/2005
	if ( ( CntStatus[0] & ( CNT_CLKEXT | CNT_GATEEXT ) ) 
		|| ( CntStatus[2] & ( CNT_CLKEXT | CNT_GATEEXT ) ) )
	{
		printk(KERN_ERR "Resource conflicts: DI0 or DI1 is used by counter!\n");
		return -1;
	}
	// Liming add ended
	usStatus = inb(iobase + EDGE_OFFSET);
  
	if (EventType & ADS_EVT_DI0) {
		if (RisingEdge == EVENT_EDGE_RISING)
			usStatus |= EVENT_DI0_RISING; 
		else
			usStatus &= EVENT_DI0_FALLING;
	}
  
	if (EventType & ADS_EVT_DI1) {
		if (RisingEdge == EVENT_EDGE_RISING)
			usStatus |= EVENT_DI1_RISING; 
		else
			usStatus &= EVENT_DI1_FALLING;  
	}
  
	outb(usStatus, iobase + EDGE_OFFSET); 
  
	return 0;   
}

// *****************************************************************************
// Design Notes:  
// -----------------------------------------------------------------------------
int LedEnable ( short usMode )
{
	DEBUGPRINT("[unodio - LedEnable]: entry\r\n");
  
	// Mode Description
	// LED_DISABLE  0x0  // disable diagnostic LED 
	// LED_ON       0x1  // Light on
	// LED_FAST     0x3  // fast flicker
	// LED_NORMAL   0x5  // normal flick
	// LED_SLOW     0x7  // slow flicker    
	outb(usMode, iobase + LED_OFFSET);      
	DEBUGPRINT("[unodio - LedEnable]: leave\r\n");
	return 0;
}

// *****************************************************************************
// Design Notes:  
// -----------------------------------------------------------------------------
int LedToggle ( unsigned short usMode,unsigned short usMask )
{
	//printk("mode = 0x%x,0x%x", usMode,usMask);
	unsigned short usStatus = 0;
	DEBUGPRINT("[unodio - LedToggle]: entry\r\n");

 	usStatus = inb(iobase + LED_OFFSET1);
	
	usMode = (usMode & usMask) | (usStatus & ~(usMask));
	outb(usMode, iobase + LED_OFFSET1 );      

	DEBUGPRINT("[unodio - LedEnable]: leave\r\n");
	return 0;
}

// *****************************************************************************
// Design Notes:  
// -----------------------------------------------------------------------------
int BuzzerEnable ( short usMode )
{
	DEBUGPRINT("[unodio - BuzzerEnable]: entry\r\n");
  
	// Mode Description
	// BEEP_DISABLE  0x0 // disable alarm buzzer
	// BEEP_ON       0x1 // beep on
	// BEEP_SHORT    0x3 // short beep
	// BEEP_MIDDLE   0x5 // normal beep
	// BEEP_LONG     0x7 // long beep
	outb(usMode, iobase + BUZZER_OFFSET);      

	DEBUGPRINT("[unodio - BuzzerEnable]: leave\r\n");
	return 0;       
}

// *****************************************************************************
// Design Notes:  
// -----------------------------------------------------------------------------
int WritePortByte ( 
	short offset, 
	short ByteData )
{
	// Liming added, 8/2/2005
	// test channel is busy or not
	if ( offset == 1)
	{
		if ( ( CntStatus[0] & CNT_OUTPUT ) 
			|| (CntStatus[1] & CNT_OUTPUT ) 
			|| ( CntStatus[2] & CNT_OUTPUT ) )
		{
			printk( KERN_ERR "Warning: Output pins are used by counter!\n");
		}
	}
	// Liming add ended
	outb(ByteData, iobase + offset);  
	return 0;  
}

// *****************************************************************************
// Design Notes:  
// -----------------------------------------------------------------------------
int ReadPortByte ( short offset, short* ByteData )
{
	// Liming added, 8/2/2005
	int data;
	data = CNT_CLKEXT | CNT_GATEEXT;
	// test channel is busy or not
	if ( offset == 0 )
	{
		if ( ( CntStatus[0] & data ) 
			|| (CntStatus[1] & data ) 
			|| ( CntStatus[2] & data ) )
		{
			printk( KERN_ERR "Warning: Input pins are used by counter!\n");
		}
	}
	// Liming add ended
	*ByteData = inb(iobase + offset);   
	return 0;
}


// *****************************************************************************
// Design Notes:  interrupt service method
// -----------------------------------------------------------------------------
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,0)
void unodio_interrupt (
	int    irq, 
	void   *dev_id,
	struct pt_regs *regs )   
#elif LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 19)
irqreturn_t unodio_interrupt (
	int    irq, 
	void   *dev_id,
	struct pt_regs *regs ) 
#else
irqreturn_t unodio_interrupt (
	int    irq, 
	void   *dev_id )     
#endif
{
	////unsigned long flags;
	 hdflag = 0;
	 g_evtstatus = 0;
  
	////save_flags(flags);
	DEBUGPRINT("->interrupt_service_unodio(%d,...)\n",irq);
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,0)
	disable_irq( irq );
#endif

	hdflag = inb(iobase + INT_STATUS_OFFSET);
	DEBUGPRINT("hdflag = 0x%x\n",hdflag);
	if ( hdflag == 0 )
	{
		DEBUGPRINT("The interrupt is not occurred!\n");
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,0)
		goto out;
#else
		return IRQ_NONE;
#endif	
	}
	if ((hdflag & 0x1) != 0 ) 
	{
		g_evtstatus |= EVT_DI0;
		DEBUGPRINT("get DI0 interrupt! %d\n", g_evtstatus);
		outb(0x01, iobase + INT_STATUS_OFFSET);
	}

	if ((hdflag & 0x2) != 0) 
	{
		g_evtstatus |= EVT_DI1;
		DEBUGPRINT("get DI1 interrupt! %d\n", g_evtstatus);
		outb(0x02, iobase + INT_STATUS_OFFSET);
	}
  
	if ((hdflag & 0x4) != 0 ) 
	{
		DEBUGPRINT("get COUNTER0 interrupt!\n");
		g_evtstatus |= EVT_CNT0;
		outb(0x04, iobase + INT_STATUS_OFFSET);
	}

	if ((hdflag & 0x8) != 0 ) 
	{
		DEBUGPRINT("get COUNTER1 interrupt!\n");
		g_evtstatus |= EVT_CNT1;
		outb(0x08, iobase + INT_STATUS_OFFSET);
	}
	g_con = 1;
	wake_up_interruptible(&adv_queue);
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,0)
out:	enable_irq( irq );
#else
	return IRQ_HANDLED;
#endif
}


void uno_function_init(void)
{
	pGetPowerStatus = GetPowerStatus;
	pDioReadPortByte = DioReadPortByte;
	pDioWritePortByte = DioWritePortByte;
	pDioReadBit = DioReadBit;
	pDioWriteBit = DioWriteBit;
	pDioGetCurrentDOByte = DioGetCurrentDOByte;
	pDioGetCurrentDOBit = DioGetCurrentDOBit;
	pCounterClockConfig = CounterClockConfig;
	pCounterConfig = CounterConfig;
	pCounterGateSet = CounterGateSet;
	pCounterOutputMonitor = CounterOutputMonitor;
	pCounterStartIO = CounterStartIO;
	pCounterTimerStart = CounterTimerStart;
	pCounterReadIO = CounterReadIO;
	pCounterEventStart = CounterEventStart;
	pCounterEventRead32 = CounterEventRead32;
	pCounterEventRead = CounterEventRead;
	pfreq_thread = freq_thread;
	pCounterFreqStart  = CounterFreqStart;
	pCounterFreqRead  = CounterFreqRead;
	pCounterPulseStart = CounterPulseStart;
	pEventEnable = EventEnable;
	pCounterReset = CounterReset;
	pEventConfig  = EventConfig;
	pLedEnable = LedEnable;
	pLedToggle = LedToggle;
	pBuzzerEnable = BuzzerEnable; 
	pWritePortByte = WritePortByte;
	pReadPortByte   = ReadPortByte ;
	punodio_interrupt  = unodio_interrupt;
	return;
};

