
/* #############################################################################
   *****************************************************************************
                  Copyright (c) 2007, Advantech eAutomation Division
                              Advantech Co., Ltd.
                               UNO Linux DIO driver
        THIS IS AN UNPUBLISHED WORK CONTAINING CONFIDENTIAL AND PROPRIETARY
                 INFORMATION WHICH IS THE PROPERTY OF ADVANTECH AUTOMATION CORP.
      ANY DISCLOSURE, USE, OR REPRODUCTION, WITHOUT WRITTEN AUTHORIZATION FROM
                 ADVANTECH AUTOMATION CORP., IS STRICTLY PROHIBITED.
   *****************************************************************************
   #############################################################################
   File:            uno_dio.h
   Description:     the headfile for driver
   Status:          works

   Permission to use, copy, modify, and distribute this software and its
   documentation for any purpose and without fee is hereby granted, provided
   that the above copyright notice appear in all copies and that both that
   copyright notice and this permission notice appear in supporting
   documentation.  This software is provided "as is" without express or
   implied warranty.
  ----------------------------------------------------------------------------*/
#include "../include/unodio_ioctl.h"
#ifndef __UNO_DIO_H
#define __UNO_DIO_H
//###########################################################
extern int iobase;  

#define ADVANTECH_VENDOR	0x13fe	/* Advantech PCI vendor ID */
#define UNO2050                 0x2E50
#define UNO2679                 0x2E79
#define UNO1171                 0x1171

//they are for PCI DIO
#define DEFAULT_BASE_ADDRESS	0x200
#define DEFAULT_IO_RANGE	0x24

//UNODIO Register Offset 
#define DI_OFFSET                0x00   // digital input register
#define DO_OFFSET                0x01   // digital output register
#define DI_INT_OFFSET            0x02   // interrupt enable control for DI0&DI1
#define EDGE_OFFSET              0x03   // interrupt edge control for DI0 & DI1
#define INT_STATUS_OFFSET        0x07   // interrupt flag & clear regsiter
#define CONT_OFFSET              0x08   // counter 0 register
#define TIMEBASE_OFFSET          0x0f   // time base offset
#define CTRL_OFFSET              0x0b   // counter control register
#define INTL_START_OFFSET        0x0c   // counter 0 start control
#define COUNTER_MONITOR_OFFSET   0x0c   // counter 0 output monitor status
#define CASCADE_OFFSET           0x0f   // cascaded mode offset
#define CONT_SETTING_OFFSET      0x0e
#define CONT_A_OFFSET            0x0e
#define CONT_B_OFFSET            0x0f
#define LED_OFFSET               0x10
#define LED_OFFSET1              0x14
#define	PLED_OFFSET              0x14
#define BUZZER_OFFSET            0x11
#define POWER_OFFSET			 0x18
//End UNODIO Register Offset 

//UNODIO Register Offset for UNO-2679
#define DO_OFFSET_2679                0x00   // digital output register
#define DI_OFFSET_2679                0x02   // digital input register DI0~DI7
#define DI1_OFFSET_2679               0x03   // digital input register for DI8 
#define DI_INT_OFFSET_2679            0x04   // interrupt enable control for DI0~DI7
#define DI1_INT_OFFSET_2679           0x05   // interrupt enable control for DI8
#define INT_STATUS_OFFSET_2679        0x08   // interrupt flag & clear regsiter for DI0~DI7
#define INT1_STATUS_OFFSET_2679       0x09   // interrupt flag & clear regsiter for DI8
//End UNODIO Register Offset for UNO-2679


//interrupt enable for DI offset (02)
#define INT_DI0_EN               0x01 // using or (bit 0) set 1
#define INT_DI0_DIS              0xfe // using and(bit 0) set 0

#define INT_DI1_EN               0x02 // using or (bit 1) set 1
#define INT_DI1_DIS              0xfd // using and(bit 1) set 0

#define INT_DI2_EN               0x04 // using or (bit 2) set 1
#define INT_DI2_DIS              0xfb // using and(bit 2) set 0

#define INT_DI3_EN               0x08 // using or (bit 3) set 1
#define INT_DI3_DIS              0xf7 // using and(bit 4) set 0

#define INT_DI4_EN               0x10 // using or (bit 4) set 1
#define INT_DI4_DIS              0xef // using and(bit 4) set 0

#define INT_DI5_EN               0x20 // using or (bit 5) set 1
#define INT_DI5_DIS              0xdf // using and(bit 5) set 0

#define INT_DI6_EN               0x40 // using or (bit 6) set 1
#define INT_DI6_DIS              0xbf // using and(bit 6) set 0

#define INT_DI7_EN               0x80 // using or (bit 7) set 1
#define INT_DI7_DIS              0x7f // using and(bit 7) set 0

#define INT_DI8_EN               0x01 // using or (bit 8) set 1
#define INT_DI8_DIS              0xfe // using and(bit 8) set 0

//offset 0x0c (INTL_START_OFFSET)
#define START_COUNTER            0x01
#define ONE_PULSE_COUNTER        0x02
#define MONITOR_COUNTER          0x10
#define EXTCLK_OFF               0xf8
//offset (0c)INTL_START_OFFSET 
#define CTR_COUNTER_START        0x01 // using or  (bit 0) set 1
#define CTR_COUNTER_STOP         0xfe // using and (bit 0) set 0
//offset (0c)COUNTER_MONITOR_OFFSET
#define CTR_COUNTER_MONITOR      0x10 // using or (bit 4) set 1   
#define CTR_COUNTER_NON_MONITOR  0xef // using and (bit 4)set 0


// data
#define INIT_0XFF	0xff

// interrupt trigger edge (offset 3)
#define EVENT_DI0_RISING   0x01  // using or (bit 0) set 1
#define EVENT_DI0_FALLING  0xfe // using and(bit 0) set 0
#define EVENT_DI1_RISING   0x02  // using or (bit 1) set 1
#define EVENT_DI1_FALLING  0xfd // using and(bit 1) set 0

#define EVENT_DI0_MASK     0x01
#define EVENT_DI1_MASK     0x02
#define EVENT_DI2_MASK     0x04
#define EVENT_DI3_MASK     0x08
#define EVENT_DI4_MASK     0x10
#define EVENT_DI5_MASK     0x20
#define EVENT_DI6_MASK     0x40
#define EVENT_DI7_MASK     0x80
#define EVENT_DI8_MASK     0x100
#define EVENT_CONTA_MASK   0x04
#define EVENT_CONTB_MASK   0x08

// interrupt flag & clear (offset 7)
#define INT_DI0_MASK             0x1
#define INT_DI1_MASK             0x2
#define INT_CNTA_MASK            0x4
#define INT_CNTB_MASK            0x8

// counter setting (0e and 0f)
#define CLOCK_EXTERNAL           0x01 // using or  (bit 0) set 1
#define CLOCK_INTERNAL           0xfe // using and (bit 0) set 0
#define GATE_EXTERNAL            0x02 // using or  (bit 1) set 1
#define GATE_INTERNAL            0xfd // using and (bit 1) set 0
#define OUTPUT_EXTERNAL          0x04 // using or  (bit 2) set 1
#define OUTPUT_INTERNAL          0xfb // using and (bit 2) set 0
#define INT_CNT_EN               0x08 // using or  (bit 3) set 1
#define INT_CNT_DIS              0xf7 // using or  (bit 3) set 1
#define TIMEBASE_MASK            0x30 // using or  (bit 4 & 5) 
#define CASCADE_ON               0x40 // using or  (bit 6) set 1
#define CASCADE_OFF              0xbf // using and (bit 6) set 0

// LED Mode
#define LED_DISABLE  0x0  // disable diagnostic LED
#define LED_ON       0x1  // Light on
#define LED_FAST     0x3  // fast flicker
#define LED_NORMAL   0x5  // normal flick
#define LED_SLOW     0x7  // slow flicker

// Buzzer Mode
#define BEEP_DISABLE  0x0 // disable alarm buzzer
#define BEEP_ON       0x1 // beep on
#define BEEP_SHORT    0x3 // short beep
#define BEEP_MIDDLE   0x5 // normal beep
#define BEEP_LONG     0x7 // long beep

// Counter channel number
#define COUNTER_0     0x0  // 16-bit counter 0
#define COUNTER_1     0x1  // 16-bit counter 1
#define COUNTER_32    0x2  // Cascaded 32-bit counter of counter 0 and counter 1

// Event Type
#define ADS_EVT_DI0     0x01   // for DI 0
#define ADS_EVT_DI1     0x02   // for DI 1
#define ADS_EVT_DI2     0x04   // for DI 2
#define ADS_EVT_DI3     0x08   // for DI 3
#define ADS_EVT_DI4     0x10   // for DI 4
#define ADS_EVT_DI5     0x20   // for DI 5
#define ADS_EVT_DI6     0x40   // for DI 6
#define ADS_EVT_DI7     0x80   // for DI 7
#define ADS_EVT_DI8     0x100  // for DI 8
#define ADS_EVT_CONT0   0x04   // for counter 0
#define ADS_EVT_CONT1   0x08   // for counter 1 or cascaded 32-bit counter

// Internal clock's time base selecting (TimerBase)
#define TIME_BASE_100K  0x00    // time base 100K Hz
#define TIME_BASE_10K   0x10    // time base 10K Hz
#define TIME_BASE_1K    0x20    // time base 1K Hz
#define TIME_BASE_100   0x30    // time base 100 Hz

// clock source selecting (for ClockPin parameter)
#define CFG_CLOCK_INTERNAL  0   // clock source from internal clock
#define CFG_CLOCK_EXTERNAL  1   // clock source from external clock
                                // Counter 0 from DI6
				// Counter 1 and cascaded 32-bit counter

// Gate tigger selecting (for GatePin paramete)
#define CFG_GATE_INTERNAL    0  // gate input from internal trigger
#define CFG_GATE_EXTERNAL    1  // gate input from external trigger
			        // Counter 0 from
				// Counter 1 and cascaded 32-bit counter

// Output pin selecting (for OutPin parameter)
#define CFG_OUTPIN_NON_DO    0  // Counter not output to DO channel
#define CFG_OUTPIN_TO_DO     1  // Counter output to DO channel
                                // Counter 0 output to DO6
 			        // Counter 1 and cascaded 32-bit counter to DO7

// In usGATEPin=CFG_GATE_INTERNAL for CTRGATE parameter's definition
#define CFG_CTR_STOP    0       // counter pause
#define CFG_CTR_START   1       // counter start or resume

// Counter output - Output pin selecting (for OutPin parameter)
#define CFG_OUTPIN_NON_MONITOR  0  // Counter not output to CTR0Out or CTR1Out
#define CFG_OUTPIN_MONITOR      1  // Counter output pin to CTR0Out or CTR1Out
#define EVENT_EDGE_RISING  	1  // interrpt trigger edge for rising edge
#define EVENT_EDGE_FALLING 	0  // interrpt trigger edge for falling edge

#define EVT_DI0    0x0001
#define EVT_DI1    0x0002
#define EVT_DI2    0x0004
#define EVT_DI3    0x0008
#define EVT_DI4    0x0010
#define EVT_DI5    0x0020
#define EVT_DI6    0x0040
#define EVT_DI7    0x0080
#define EVT_DI8    0x0100
#define EVT_CNT0   0x0004
#define EVT_CNT1   0x0008

#define GETEVTSTA_DI0           0x0001
#define GETEVTSTA_DI1           0x0002
#define GETEVTSTA_CNT0          0x0004
#define GETEVTSTA_CNT1          0x0008

// Set the flags for counter with CntStatus[3]. CntStatus[i] is the flag of 
// counter i, its value is 0x00, 0x01, 0x02, 0x04, 0x08, 0x10:
// 0x00: counter is ready
// 0x01: counter is busy 
// 0x02: counter clock is configed 
// 0x04: counter uses external clock
// 0x08: counter uses external gate
// 0x10: output pin is used by counter
#define CNT_READY	0x00
#define	CNT_BUSY	0x01
#define	CNT_CLKCFG	0x02
#define CNT_CLKEXT	0x04
#define CNT_GATEEXT	0x08
#define CNT_OUTPUT	0x10
 



#ifndef KERNEL_VERSION
#define KERNEL_VERSION(a,b,c)     ((a)*65536+(b)*256+(c))
#endif

//#define ADVANTECH_DEBUG
#ifdef ADVANTECH_DEBUG
#define DEBUGPRINT printk
#else
#define DEBUGPRINT(a, ...)
#endif
enum unodio_type {
	unodio_4channel = 0, // for UNOs ISA DIO, 4 DIO channel device
	unodio_8channel,     // for UNO-2052(E),UNO_4672,2176, 8 DIO channel
	unodio_2679,         // for UNO-2679, 2 port, 8DIO for port0 and 1 DI for port1
	unodio_1171,
} ;                                                  
struct devinfo
{
	struct devinfo * pNext;        // next device info. address
	int      DeviceNum;            // device number
	int      iobase;               // base address
	int      irq;                  // interrupt number
	int      IntID;                // interrupt indify
	short    EventType;            // event type DI0, DI1 and Counter
	short    UsageCount;           // usage counter

	short    TimeBase;             // time base config

	// for 16-bit counter
	int      OverFlow[2];         // overflow flag for event
	unsigned short PreCount[2];   // previous count for 16-bit counter
	int	 FirstCount[2];	      // first count for 16-bit counter

	// for 32-bit counter
	short    PreCountLow;         // previous count for Low (Counter A)
	short    PreCountHigh;        // previous count for High (Counter B)
	int      PreCounter32;        // previous count for 32-bit counter
	int      OverFlow32;          // over count for 32-bit counter
	int      FirstCount32;        // first count for 32-bit counter

	// frequence measure
	int      FreqOverFlow[2];     // overflow flag for freq
	short    FreqPreCount[2];     // previous count state for freq
	int      FreqPreTicks[2];     // previous ticks for freq
	int FreqPreData[2];           // previous data for freq
	// joshua add 03/02/2005
	int freq_thread_flag[2];      // flag to restore the freq thread status
	int counter;	
	unsigned long FreqPreReTicks[2];
	unsigned long FreqCurReTicks[2];
	unsigned long FreqInitData[2];
	unsigned long FreqInitTicks[2];	
	unsigned long FreqCurData[2];
	unsigned long FreqCurTicks[2];		
	//add end
};


		


extern wait_queue_head_t adv_queue;
extern int g_con;


void uno2679_function_init(void);
void uno_function_init(void);

extern int (*pGetPowerStatus )( short* powerstatus );
extern int (*pDioReadPortByte) ( short port, short* value );
extern int (*pDioWritePortByte )(short port, short mask, short state );
extern int (*pDioReadBit) (short port, short bit, short* state );
extern int (*pDioWriteBit) (short port, short bit, short state );
extern int (*pDioGetCurrentDOByte) ( short port, short* value );
extern int (*pDioGetCurrentDOBit) (short port, short bit, short* state );
extern int (*pCounterClockConfig) ( short TimeBase );
extern int (*pCounterConfig )(short counter, short ClockPin, short GatePin, short OutPin );
extern int (*pCounterGateSet) ( short counter, short CTRGate );
extern int (*pCounterOutputMonitor) ( short counter, short* state );
extern int (*pCounterStartIO) ( short counter, short InitLoCount, short InitHiCount );
extern int (*pCounterTimerStart) ( short counter, long InitHWCount );
extern unsigned short (*pCounterReadIO) ( short counter );
extern int (*pCounterEventStart) ( short counter, short GateMode );
extern int (*pCounterEventRead32) (short* overflow, unsigned long* count );
extern int (*pCounterEventRead) ( short counter, short* overflow, unsigned long * count );
extern int (*pfreq_thread)( void * param );
extern int (*pCounterFreqStart) ( short counter, short GatePeriod,short GateMode );
extern int (*pCounterFreqRead) (short counter,unsigned long *timelapse,long *cntnum,long *kernelhz );
extern int (*pCounterPulseStart) (short counter, long  period, long  UpCycle, short GateMode )  ;
extern int (*pEventEnable) (short EventType, short Enabled, short Count );
extern int (*pCounterReset )( short counter );
extern int (*pEventConfig) ( short EventType, short RisingEdge );
extern int (*pLedEnable) ( short usMode );
extern int (*pLedToggle) ( unsigned short usMode,unsigned short usMask );
extern int (*pBuzzerEnable) ( short usMode );
extern int (*pWritePortByte) ( short offset, short ByteData );
extern int (*pReadPortByte) ( short offset, short* ByteData );
extern unsigned short g_evtstatus;
extern short hdflag; 
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,0)
extern void (*punodio_interrupt)(int  irq, void   *dev_id,struct pt_regs *regs )  ; 
#elif LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 19)
extern irqreturn_t (*punodio_interrupt)(int  irq, void   *dev_id,struct pt_regs *regs ) ;
#else
extern irqreturn_t (*punodio_interrupt)(int irq, void   *dev_id ) ;    
#endif
#endif //ifdef__UNO_DIO_H

