/* #############################################################################
   *****************************************************************************
                  Copyright (c) 2007, Advantech eAutomation Division
                              Advantech Co., Ltd.
                               UNO Linux DIO driver
        This code is loosely based on uno2050 dio and unoisa dio drivers,written
         by Joshua.lan and others.
        THIS IS AN UNPUBLISHED WORK CONTAINING CONFIDENTIAL AND PROPRIETARY
           INFORMATION WHICH IS THE PROPERTY OF ADVANTECH AUTOMATION CORP.
  
      ANY DISCLOSURE, USE, OR REPRODUCTION, WITHOUT WRITTEN AUTHORIZATION FROM
                 ADVANTECH AUTOMATION CORP., IS STRICTLY PROHIBITED.
   *****************************************************************************
   #############################################################################
  
   File: 	    uno_dio_2679.c
  
   Description:     This is a DI/O driver module for Advantech UNOs.
   Status: 	    works
  
   Permission to use, copy, modify, and distribute this software and its
   documentation for any purpose and without fee is hereby granted, provided
   that the above copyright notice appear in all copies and that both that
   copyright notice and this permission notice appear in supporting
   documentation.  This software is provided "as is" without express or
   implied warranty.
  ----------------------------------------------------------------------------*/

#include <linux/init.h>
#include <linux/module.h>
#include <linux/version.h>
#include <linux/types.h>
#include <linux/errno.h>
#include <linux/kernel.h>
#include <linux/sched.h>
#include <linux/interrupt.h>
#include <linux/ioport.h>
#include <linux/ioctl.h>
#include <asm/io.h>
#include <linux/pci.h>
#include <linux/poll.h>
#include <linux/fs.h>
#include <linux/param.h>
#include <asm/uaccess.h>

#include "uno_dio.h"

// *****************************************************************************
// Design Notes:  
// -----------------------------------------------------------------------------
int DioReadPortByte_2679 ( short port, short* value )
{
	DEBUGPRINT("[unodio - DioReadPortByte]: entry\r\n");
	*value = inb(iobase + DI_OFFSET_2679+port);
	if (port == 1)
	{
		*value = (*value & 0x1);
	}
	return 0;
}


// *****************************************************************************
// Design Notes:  
//-----------------------------------------------------------------------------
int DioWritePortByte_2679 (
	short port, 
	short mask, 
	short state )
{
	short ReadBack;
  
	DEBUGPRINT("[unodio - DioWritePortByte]: entry\r\n");
	state = state & 0xff;
	mask = mask & 0xff;    
  
	// read back the current state
	ReadBack = inb(iobase + DO_OFFSET_2679);
   
	state = (state & mask) | (ReadBack & ~(mask));
 
	//  write the desired output state to the hardware 
	outb(state, iobase + DO_OFFSET_2679);
	DEBUGPRINT("[unodio - DioWritePortByte]: leave\r\n");
  
	return 0;
}

// *****************************************************************************
// Design Notes:  
// -----------------------------------------------------------------------------
int DioReadBit_2679 (
	short port, 
	short bit, 
	short* state )
{
	short wMask;
  
	DEBUGPRINT("[unodio - DioReadBit]: entry\r\n");
  
	*state = inb(iobase + DI_OFFSET_2679);

	if ( bit <= 7 ) 
	{
		wMask = 0x01;
		wMask = wMask << bit;
		*state = *state & wMask;
    
		if ( *state )
			*state = 1;        
		else
			*state = 0;     
	}
  
	DEBUGPRINT("[unodio - DioReadBit]: leave\r\n");
  
	return 0;
}


// *****************************************************************************
// Design Notes:  
// -----------------------------------------------------------------------------
int DioWriteBit_2679 (
	short port, 
	short bit, 
	short state )
{
	short mask;
	short ReadBack;
  
	DEBUGPRINT("[unodio - DioWriteBit]: entry\r\n");
	mask = 0x01;
	mask = mask << bit;
	state = state << bit;
  
	// read back the current state
	ReadBack = inb(iobase + DO_OFFSET_2679);
	state = (state & mask) | (ReadBack & ~(mask));
  
	//  write the desired output state to the hardware 
	outb(state, iobase + DO_OFFSET_2679);
  
	DEBUGPRINT("[unodio - DioWriteBit]: leave\r\n");
  
	return 0;
}

// *****************************************************************************
// Design Notes:  
// ----------------------------------------------------------------------------
int DioGetCurrentDOByte_2679 ( short port, short* value )
{
	DEBUGPRINT("[unodio - DioGetCurrentDOByte]: entry\r\n");

	*value = inb(iobase + DO_OFFSET_2679);
  
	return 0;
}

// *****************************************************************************
// Design Notes:  
// -----------------------------------------------------------------------------
int DioGetCurrentDOBit_2679 (
	short port, 
	short bit, 
	short* state )
{
	short mask;
	int ret;
  
	DEBUGPRINT("[unodio - DioGetCurrentDOBit]: entry\r\n");
  
	ret = inb(iobase + DO_OFFSET_2679);
	*state = ret;
  
	if ( bit <= 7 )
 	{
		mask = 0x01;
		mask = mask << bit;
		*state = *state & mask;
    
		if (*state)
			*state = 1;        
		else
			*state = 0;            
	}
  
  	DEBUGPRINT("[unodio - DioGetCurrentDOBit]: leave\r\n");
  
	return 0;
}

// *****************************************************************************
// Design Notes:  
// -----------------------------------------------------------------------------
int EventEnable_2679 (
	short EventType, 
	short Enabled, 
	short Count )
{
	unsigned short usStatusDI, usStatusDI1;
  
	DEBUGPRINT("->EventEnable\n");
    
	// read back some hardware values
	usStatusDI   = inb(iobase + DI_INT_OFFSET_2679);
	usStatusDI1 = inb(iobase + DI1_INT_OFFSET_2679);     
	if (Enabled) 
	{ 
          
		if (EventType & EVENT_DI0_MASK) 
		{               
			usStatusDI |= INT_DI0_EN;
    		}
		if (EventType & EVENT_DI1_MASK)        
			usStatusDI |= INT_DI1_EN; 
 
		if (EventType & EVENT_DI2_MASK)                
			usStatusDI |= INT_DI2_EN;
    
		if (EventType & EVENT_DI3_MASK)        
			usStatusDI |= INT_DI3_EN;

		if (EventType & EVENT_DI4_MASK)                
			usStatusDI |= INT_DI4_EN;
    
		if (EventType & EVENT_DI5_MASK)        
			usStatusDI |= INT_DI5_EN;

		if (EventType & EVENT_DI6_MASK)                
			usStatusDI |= INT_DI6_EN;
    
		if (EventType & EVENT_DI7_MASK)        
			usStatusDI |= INT_DI7_EN;                      
    
		outb(usStatusDI, iobase + DI_INT_OFFSET_2679);
    
		if (EventType & EVENT_DI8_MASK)
			outb((usStatusDI1 | EVENT_DI8_MASK), iobase + DI1_INT_OFFSET_2679);  
    
	}
	else 
	{
		if (EventType & EVENT_DI0_MASK)
			usStatusDI &= INT_DI0_DIS;
		if (EventType & EVENT_DI1_MASK)
			usStatusDI &= INT_DI1_DIS;
		if (EventType & EVENT_DI2_MASK)
			usStatusDI &= INT_DI2_DIS;
		if (EventType & EVENT_DI3_MASK)
			usStatusDI &= INT_DI3_DIS;
		if (EventType & EVENT_DI4_MASK)
			usStatusDI &= INT_DI4_DIS;
		if (EventType & EVENT_DI5_MASK)
			usStatusDI &= INT_DI5_DIS;
		if (EventType & EVENT_DI6_MASK)
			usStatusDI &= INT_DI6_DIS;
		if (EventType & EVENT_DI7_MASK)
			usStatusDI &= INT_DI7_DIS;

		outb(usStatusDI, iobase + DI_INT_OFFSET_2679);
    
		if (EventType & EVENT_DI8_MASK)
			outb((usStatusDI1  & INT_DI8_DIS), iobase + DI1_INT_OFFSET_2679);  
	}            
  
	return 0; 
}



// *****************************************************************************
// Design Notes:  
// -----------------------------------------------------------------------------
int WritePortByte_2679 ( 
	short offset, 
	short ByteData )
{
	// Liming added, 8/2/2005
	outb(ByteData, iobase + offset);  
	return 0;  
}

// *****************************************************************************
// Design Notes:  
// -----------------------------------------------------------------------------
int ReadPortByte_2679 ( short offset, short* ByteData )
{
	*ByteData = inb(iobase + offset);   
	return 0;
}



// *****************************************************************************
// Design Notes:  interrupt service method
// -----------------------------------------------------------------------------

#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,0)
void unodio_interrupt_2679 (
	int    irq, 
	void   *dev_id,
	struct pt_regs *regs )   
#elif LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 19)
irqreturn_t unodio_interrupt_2679 (
	int    irq, 
	void   *dev_id,
	struct pt_regs *regs ) 
#else
irqreturn_t unodio_interrupt_2679 (
	int    irq, 
	void   *dev_id )     
#endif
{
	unsigned short hdflag  = 0;
	unsigned short hdflag1 = 0; 

	DEBUGPRINT("->interrupt_service_unodio(%d,...)\n",irq);
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,0)
	disable_irq( irq );
#endif

	hdflag  = inb(iobase + INT_STATUS_OFFSET_2679);
	hdflag1 = inb(iobase + INT1_STATUS_OFFSET_2679);
	if ( hdflag == 0 && hdflag1 == 0)
	{
		DEBUGPRINT("The interrupt is not occurred!\n");
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,0)
		goto out;
#else
		return IRQ_NONE;
#endif	
	}
	if ((hdflag & 0x1) != 0 ) 
	{
		g_evtstatus |= EVT_DI0;
		DEBUGPRINT("get DI0 interrupt! %d\n", g_evtstatus);
		outb(0x01, iobase + INT_STATUS_OFFSET_2679);
	}

	if ((hdflag & 0x2) != 0) 
	{
		g_evtstatus |= EVT_DI1;
		DEBUGPRINT("get DI1 interrupt! %d\n", g_evtstatus);
		outb(0x02, iobase + INT_STATUS_OFFSET_2679);
	}
  
	if ((hdflag & 0x4) != 0 ) 
	{
		DEBUGPRINT("get COUNTER0 interrupt!\n");
		g_evtstatus |= EVT_DI2;
		outb(0x04, iobase + INT_STATUS_OFFSET_2679);
	}

	if ((hdflag & 0x8) != 0 )
	{
		DEBUGPRINT("get COUNTER1 interrupt!\n");
		g_evtstatus |= EVT_DI3;
		outb(0x08, iobase + INT_STATUS_OFFSET_2679);
	}
	
	if ((hdflag & 0x10) != 0 )
	{
		DEBUGPRINT("get COUNTER1 interrupt!\n");
		g_evtstatus |= EVT_DI4;
		outb(0x08, iobase + INT_STATUS_OFFSET_2679);
	}
	
	if ((hdflag & 0x20) != 0 )
	{
		DEBUGPRINT("get COUNTER1 interrupt!\n");
		g_evtstatus |= EVT_DI5;
		outb(0x08, iobase + INT_STATUS_OFFSET_2679);
	}
	if ((hdflag & 0x40) != 0 )
	{
		DEBUGPRINT("get COUNTER1 interrupt!\n");
		g_evtstatus |= EVT_DI6;
		outb(0x08, iobase + INT_STATUS_OFFSET_2679);
	}
	
	if ((hdflag & 0x80) != 0 )
	{
		DEBUGPRINT("get COUNTER1 interrupt!\n");
		g_evtstatus |= EVT_DI7;
		outb(0x08, iobase + INT_STATUS_OFFSET_2679);
	}
	if ((hdflag1 & 0x1) != 0 )
	{
		DEBUGPRINT("get COUNTER1 interrupt!\n");
		g_evtstatus |= EVT_DI8;
		outb(0x08, iobase + INT1_STATUS_OFFSET_2679);
	}
	g_con = 1;
	wake_up_interruptible(&adv_queue);
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,0)
out:	enable_irq( irq );
#else
	return IRQ_HANDLED;
#endif
}




void uno2679_function_init(void)
{
	//pGetPowerStatus = GetPowerStatus_2679;
	pDioReadPortByte = DioReadPortByte_2679;
	pDioWritePortByte = DioWritePortByte_2679;
	pDioReadBit = DioReadBit_2679;
	pDioWriteBit = DioWriteBit_2679;
	pDioGetCurrentDOByte = DioGetCurrentDOByte_2679;
	pDioGetCurrentDOBit = DioGetCurrentDOBit_2679;
	//pCounterClockConfig = CounterClockConfig_2679;
	//pCounterConfig = CounterConfig_2679;
	//pCounterGateSet = CounterGateSet_2679;
	//pCounterOutputMonitor = CounterOutputMonitor_2679;
	//pCounterStartIO = CounterStartIO_2679;
	//pCounterTimerStart = CounterTimerStart_2679;
	//pCounterReadIO = CounterReadIO_2679;
	//pCounterEventStart = CounterEventStart_2679;
	//pCounterEventRead32 = CounterEventRead32_2679;
	//pCounterEventRead = CounterEventRead_2679;
	//pfreq_thread = freq_thread_2679;
	//pCounterFreqStart  = CounterFreqStart_2679;
	//pCounterFreqRead  = CounterFreqRead_2679;
	//pCounterPulseStart = CounterPulseStart_2679;
	pEventEnable = EventEnable_2679;
	//pCounterReset = CounterReset_2679;
	//pEventConfig  = EventConfig_2679;
	//pLedEnable = LedEnable_2679;
	//pLedToggle = LedToggle_2679;
	//pBuzzerEnable = BuzzerEnable_2679; 
	pWritePortByte = WritePortByte_2679;
	pReadPortByte   = ReadPortByte_2679 ;
	punodio_interrupt  = unodio_interrupt_2679;
	return;

}
