/* #############################################################################
   *****************************************************************************
                  Copyright (c) 2009, Advantech eAutomation Division
                              Advantech Co., Ltd.
                               UNO Linux DIO driver
        This code is loosely based on uno2050 dio and unoisa dio drivers,written
         by Joshua.lan and others.
        THIS IS AN UNPUBLISHED WORK CONTAINING CONFIDENTIAL AND PROPRIETARY
           INFORMATION WHICH IS THE PROPERTY OF ADVANTECH AUTOMATION CORP.
  
      ANY DISCLOSURE, USE, OR REPRODUCTION, WITHOUT WRITTEN AUTHORIZATION FROM
                 ADVANTECH AUTOMATION CORP., IS STRICTLY PROHIBITED.
   *****************************************************************************
   #############################################################################
  
   File: 	    uno_dio_core.c   
   Author: 	    jianfeng
   Change log:      Version 1.00 <04/13/2007>
 			- Initial version 
  		    Version 1.10 <12/18/2008>
 			- support UNO-4672 and UNO-2679
  		    Version 1.11 <05/06/2010>
      			- to support FC10,FC11,FC12,SUSE 11.2 Madriva 2010

/////////////////////////////////////////////////////////////////////////////////
   Discarded Change log:      Version 1.10 <01/27/2005>
  			- Updated the event handling to support poll and select 
                          system call
  			- Moved all the floating opeartion from kernel mode to 
                          user level
                    Version 2.00 <02/02/2005>
			- Merge the code of kernel 2.4 and 2.6
			- Support kernel 2.4/2.6			
		    Version 2.10 <03/10/2005>
			- Update the frequency measurement function
		    Version 2.11 <06/14/2005> Liming
			- Update for the different definitions of HZ in Linux
 			  Kernel 2.4.x and 2.6.x
		    Version 2.12 <8/5/2005> Liming
			- Update for the management of device resources	
		    Version 2.13 <02/22/2006>
			- Disable the debug message in ISR
		    Version 2.14 <08/17/2006> Lipeng WangMao
                        - Fix the judgement for return value of pci_register_driver
                        - Fix the frequency calculation erro
  
   Description:     This is a DI/O driver module for Advantech UNOs.
   Status: 	    works
  
   Permission to use, copy, modify, and distribute this software and its
   documentation for any purpose and without fee is hereby granted, provided
   that the above copyright notice appear in all copies and that both that
   copyright notice and this permission notice appear in supporting
   documentation.  This software is provided "as is" without express or
   implied warranty.
  ----------------------------------------------------------------------------*/

#include <linux/init.h>
#include <linux/module.h>
#include <linux/version.h>
#include <linux/types.h>
#include <linux/errno.h>
#include <linux/kernel.h>
#include <linux/sched.h>
#include <linux/interrupt.h>
#include <linux/ioport.h>
#include <linux/ioctl.h>
#include <asm/io.h>
#include <linux/pci.h>
#include <linux/poll.h>
#include <linux/fs.h>
#include <linux/param.h>
#include <asm/uaccess.h>
#include "uno_dio.h"
#define ADVANTECH_DIO_VER               "1.11"
#define ADVANTECH_DIO_DATE              "05/06/2010" 


int iobase = DEFAULT_BASE_ADDRESS;
int io_range = DEFAULT_IO_RANGE;
int irqr = 7;
int major = 0;
int advhdbase = 0x000FE0C1;
int adv_range = 32;
static unsigned char * uc_ptaddr;
static int dio_type = 0;
static int is_pci;
unsigned short g_evtstatus = 0;
short hdflag = 0;
struct pci_dev *advdev;
wait_queue_head_t adv_queue;
// *****************************************************************************
// Design Notes:  
// -----------------------------------------------------------------------------
static int unodio_reset ( void )
{
	DEBUGPRINT("unodio EDBG: unodio_reset(...)\n");
	return 0;
}


// *****************************************************************************
// Design Notes:  
//	unodio_write:
//	file: file handle to the watchdog
//	buf: buffer to write (unused as data does not matter here 
//	count: count of bytes
//	ppos: pointer to the position to write. No seeks allowed
// -----------------------------------------------------------------------------
static ssize_t unodio_write (
	struct file *file, 
	const char *buf, 
	size_t count, 
	loff_t *ppos )
{
	DEBUGPRINT("in write()\n");	
	return 0;
}

// *****************************************************************************
// Design Notes:  
//	unodio_read:
//	file: file handle to the watchdog board
//	buf: buffer to write 1 byte into
//	count: length of buffer
//	ptr: offset (no seek allowed)
// -----------------------------------------------------------------------------
static ssize_t unodio_read (
	struct file *file, 
	char *buf, 
	size_t count, 
	loff_t *ptr )
{
	DEBUGPRINT("in read()\n");
	/*
	g_con = 0;
	if ( wait_event_interruptible( adv_queue, g_con ))
	{
		DEBUGPRINT("wait_event_interruptible() error!\n");
		return -ERESTARTSYS;
	}
	*/
	return 0;
}

static unsigned int unodio_poll ( struct file * file, poll_table * wait )
{
	DEBUGPRINT("unodio_poll()++\n");		
	poll_wait( file, &adv_queue, wait);
	if ( g_con )
	{
		g_con = 0;
		//schedule_timeout(2);
		//schedule();
		//add_wait_queue( &adv_queue, wait )
		return POLLPRI;
	}
	DEBUGPRINT("unodio_poll()--");
	return 0;
}

// *****************************************************************************
// Design Notes:  
//	unodio_ioctl:
//	inode: inode of the device
//	file: file handle to the device
//	cmd: watchdog command
//	arg: argument pointer
// -----------------------------------------------------------------------------
static int unodio_ioctl (
	struct inode *inode, 
	struct file *file, 
	unsigned int cmd, 
	unsigned long arg )
{
	unsigned short sdata[ 8 ];
	int idata[ 8 ];
	long ldata[ 8 ];
	int options, ret = 0,err=0;  // ret is added for return value of function

	DEBUGPRINT("in ioctl()\n");
	
	if ( _IOC_TYPE(cmd) != UNO_DIO_IOC_MAGIC ) return -EINVAL;
	if ( _IOC_NR(cmd) > UNO_DIO_IOC_MAXNR )	return -EINVAL;

	switch ( cmd ) 
	{
	case GETEVENT:
		if ( get_user( options, (int *)arg))
			return -EFAULT;
		put_user(g_evtstatus, (unsigned long *)arg);
		if ( options & GETEVTSTA_DI0 )
		{
			g_evtstatus &= ~EVT_DI0;;
		}
		if ( options & GETEVTSTA_DI1 )
		{
			g_evtstatus &= ~EVT_DI1;
		}
		if ( options & GETEVTSTA_CNT0 )
		{
			g_evtstatus &= ~EVT_CNT0;
		}
		if ( options & GETEVTSTA_CNT1 )
		{
			g_evtstatus &= ~EVT_CNT1;			
		}
	break;
	
	case GETPOWER:
		ret = (*pGetPowerStatus)( &sdata[ 0 ] );
		err = copy_to_user((void*)arg, sdata, sizeof(short));
	break;
	case DIO_READ_PORT_BYTE:
		err = copy_from_user(sdata,(void *)arg,2*sizeof(short));
		ret = (*pDioReadPortByte)(sdata[0], &sdata[1]);
		err = copy_to_user((void *)arg, sdata, 2*sizeof(short));
	break;
	case DIO_WRITE_PORT_BYTE:
		err = copy_from_user(sdata, (void *)arg, 2*sizeof(short));
		ret = (*pDioWritePortByte)(0, sdata[0], sdata[1]);
	break;
	case DIO_READ_BIT:
		err = copy_from_user(sdata, (void *)arg, sizeof(short));
		ret = (*pDioReadBit)(0, sdata[0], &sdata[1]);
		err = copy_to_user((void *)arg, sdata, 2*sizeof(short));
	break;
	case DIO_WRITE_BIT:
		err = copy_from_user(sdata, (void *)arg, 2*sizeof(short));
		ret = (*pDioWriteBit)(0, sdata[0], sdata[1]);
	break;
	case DIO_GET_DO_BYTE:
		ret = (*pDioGetCurrentDOByte)(0, sdata);
		err = copy_to_user((void *)arg, sdata, sizeof(short));
	break;
	case DIO_GET_DO_BIT:
		err = copy_from_user(sdata, (void *)arg, sizeof(short));
		ret = (*pDioGetCurrentDOBit)(0, sdata[0], &sdata[1]);
		err = copy_to_user((void *)arg, sdata, 2*sizeof(short));
	break;
	case COUNTER_TIMER_START:
		err = copy_from_user(ldata, (void *)arg, 2 * sizeof(long));
		ret = (*pCounterTimerStart)((short)ldata[0], ldata[1]);
	break;
	case COUNTER_EVENT_START:
		err = copy_from_user(sdata, (void *)arg, 2*sizeof(short));
		ret = (*pCounterEventStart)(sdata[0], sdata[1]);
	break;
	case COUNTER_EVENT_READ:
		err = copy_from_user(idata, (void *)arg, sizeof(long));
		ret = (*pCounterEventRead)(
			(short)idata[0], 
			&sdata[1], 
			(unsigned long *)&idata[2]);
		idata[1] = (long)sdata[1];
		err = copy_to_user((void *)arg, idata, 3 * sizeof(long));	
	break;
	case COUNTER_FREQ_START:
		err = copy_from_user(sdata, (void *)arg, 3*sizeof(short));
		ret = (*pCounterFreqStart)(sdata[0], sdata[1], sdata[2]);
	break;
	case COUNTER_FREQ_READ:
		err = copy_from_user(ldata, (void *)arg, 4 * sizeof(long));
		ret = (*pCounterFreqRead)(
			ldata[0], 
			&ldata[1], 
			&ldata[2],
			&ldata[3]);
		err = copy_to_user((void *)arg, ldata, 4 * sizeof(long));
	break;
	case COUNTER_PULSE_START:
		err = copy_from_user(ldata, (void *)arg, 4*sizeof(long));
		ret = (*pCounterPulseStart)(
			(short)ldata[0], 
			(long)ldata[1], 
			(long)ldata[2], 
			(short)ldata[3]);
	break;
	case COUNTER_RESET:
		err = copy_from_user(sdata, (void *)arg, sizeof(short));
		ret = (*pCounterReset)(sdata[0]);
	break;
	case COUNTER_CLOCK_CFG:
		err = copy_from_user(sdata, (void *)arg, sizeof(short));
		ret = (*pCounterClockConfig)(sdata[0]);
	break;
	case COUNTER_CFG:
		err = copy_from_user(sdata, (void *)arg, 4*sizeof(short));
		ret = (*pCounterConfig)(sdata[0], sdata[1], sdata[2], sdata[3]);
	break;
	case COUNTER_GATE_SET:
		err = copy_from_user(sdata, (void *)arg, 2*sizeof(short));
		ret = (*pCounterGateSet)(sdata[0], sdata[1]);
	break;
	case COUNTER_MONITOR:
		err = copy_from_user(sdata, (void *)arg, sizeof(short));
		ret = (*pCounterOutputMonitor)(sdata[0], &sdata[1]);
		err = copy_to_user((void *)arg, sdata, 2*sizeof(short));
	break;
	case WRITE_PORT_BYTE:
		err = copy_from_user(sdata, (void *)arg, 2*sizeof(short));
		ret = (*pWritePortByte)(sdata[0], sdata[1]);
	break;
	case READ_PORT_BYTE:
		err = copy_from_user(sdata, (void *)arg, sizeof(short));
		ret = (*pReadPortByte)(sdata[0], &sdata[1]);
		err = copy_to_user((void *)arg, sdata, 2*sizeof(short));
	break;
	case EVENT_CFG:
		err = copy_from_user(sdata, (void *)arg, 2*sizeof(short));
		ret = (*pEventConfig)(sdata[0], sdata[1]);
	break;
	case EVENT_ENABLE:
		err = copy_from_user(sdata, (void *)arg, 3*sizeof(short));
		ret = (*pEventEnable)(sdata[0], sdata[1], sdata[2]);
	break;
	case LED_ENABLE:
		err = copy_from_user(sdata, (void *)arg, sizeof(short));
		ret = (*pLedEnable)(sdata[0]);
	break;
	case LED_TOGGLE:
		err = copy_from_user(sdata, (void *)arg, 2*sizeof(short));
		ret = (*pLedToggle)(sdata[0], sdata[1]);
	break;
	case BUZZER_ENABLE:
		err = copy_from_user(sdata, (void *)arg, sizeof(short));
		ret = (*pBuzzerEnable)(sdata[0]);
	break;
	case DIO_TYPE:
		err = copy_to_user((void*)arg,&dio_type,sizeof(int));
		break;
	default:
		return -1;
	}
	return ret;
}

// *****************************************************************************
// Design Notes:  
//	unodio_open:
//	@inode: inode of device
//	@file: file handle to device
// -----------------------------------------------------------------------------
static int unodio_open (
	struct inode *inode, 
	struct file *file )
{
	DEBUGPRINT("in open()\n");
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,0) 
	MOD_INC_USE_COUNT;
#endif
	
	return 0;
}

// *****************************************************************************
// Design Notes:  
//	unodio_close:
//	inode: inode to board
//	file: file handle to board
// -----------------------------------------------------------------------------
static int unodio_release (
	struct inode *inode, 
	struct file *file )
{
	DEBUGPRINT("in release\n");
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,0)
	MOD_DEC_USE_COUNT;
#endif
	// EventEnable( EVENT_DI0_MASK|EVENT_DI1_MASK|EVENT_CONTA_MASK|EVENT_CONTB_MASK, 0, 1);
	return 0;
}

// *****************************************************************************
// Design Notes:  
//	Kernel Interfaces
// -----------------------------------------------------------------------------
static struct file_operations unodio_fops = {
	owner:		THIS_MODULE,
	read:		unodio_read,
	write:		unodio_write,
	poll:		unodio_poll,
	ioctl:		unodio_ioctl,
	open:		unodio_open,
	release:	unodio_release,
};

// ****************************************************************************
// Design Notes:
// -----------------------------------------------------------------------------
static int unodio_init_one ( 
	struct pci_dev *dev,
	const struct pci_device_id *ent )
{
	static int dev_count = 0;
	int ret = -EIO;

	if (ent->device == UNO2679)
	{
		uno2679_function_init();
		dio_type = unodio_2679;
	}
	else
	{
		uno_function_init();
		if (ent->device == UNO1171)
		{
			dio_type = unodio_1171;			
		}
		else
		{ 
			dio_type = unodio_8channel;
		}
	}

  
	dev_count++;
  	advdev = dev;
	if (dev_count > 1) 
	{
		printk ("This driver only supports one device.\n");
		return -ENODEV;
	}
  
	irqr = dev->irq;
	iobase = pci_resource_start (dev, 0);
	printk("=====================================================\n");
	printk("     DIO driver for Advantech UNOs  V%s [%s]\n", 
		ADVANTECH_DIO_VER, ADVANTECH_DIO_DATE);
	printk("     IO base address: 0x%XH\n", iobase);
	printk("     IRQ number:      %d\n", irqr );
	printk("     Advantech eAutomation Division.\n");
	printk("=====================================================\n");
  
	if (pci_enable_device (dev))
		goto out;
  		
	init_waitqueue_head( &adv_queue );

	if (request_region (iobase, 16, "unodio") == NULL) 
	{
		printk ("I/O %d is not free.\n", iobase);
		goto out;
	}
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 23)
	if ( request_irq (irqr, punodio_interrupt, IRQF_SHARED,
		"unodio", 
		&unodio_fops)) 
#else
	if ( request_irq (irqr, punodio_interrupt, SA_SHIRQ,
		"unodio", 
		&unodio_fops)) 
#endif
	{
		printk ("IRQ %d is not free.\n", irqr);
		goto out_irq;
	}
	ret = 0;
	return ret;
out:
	release_region(iobase, 16);
	return ret;
 
out_irq:
	free_irq(irqr, &unodio_fops);
	goto out;
}

// *****************************************************************************
// Design Notes:  
// -----------------------------------------------------------------------------
static void unodio_remove_one ( struct pci_dev *pdev )
{
	/* here we assume only one device will ever have
	* been picked up and registered by probe function */
	free_irq(irqr, &unodio_fops);
	release_region(iobase, 16);
}

// *****************************************************************************
// Design Notes:  interrupt service method
// unsigned int vendor, device;         /* Vendor and device ID or PCI_ANY_ID */
// unsigned int subvendor, subdevice;   /* Subsystem ID's or PCI_ANY_ID */
// unsigned int class, class_mask;      /* (class,subclass,prog-if) triplet */
// unsigned long driver_data;           /* Data private to the driver */
// -----------------------------------------------------------------------------
static struct pci_device_id unodio_pci_tbl[] = {
	{ ADVANTECH_VENDOR, UNO2050, PCI_ANY_ID, PCI_ANY_ID, },
	{ ADVANTECH_VENDOR, UNO2679, PCI_ANY_ID, PCI_ANY_ID, },
	{ ADVANTECH_VENDOR, UNO1171, PCI_ANY_ID, PCI_ANY_ID, },
	{ 0, } /* terminate list */
};
MODULE_DEVICE_TABLE(pci, unodio_pci_tbl);

static struct pci_driver unodio_driver = {
	name:		"unodio",
	id_table:	unodio_pci_tbl,
	probe:		unodio_init_one,
	remove:		unodio_remove_one,
};

// *****************************************************************************
// Design Notes:  
//	unodio_cleanup:
//	Unload our module
// -----------------------------------------------------------------------------
void unodio_cleanup ( void )
{
	pci_unregister_driver( &unodio_driver );
	if (is_pci==0)
	{
		free_irq(irqr, &unodio_fops);
		release_region( iobase, io_range );
	}
	unregister_chrdev( major, "unodio" );
}

// *****************************************************************************
// Design Notes:  
// 	unodio_init:
//      init our Module
// -----------------------------------------------------------------------------
int unodio_init ( void )
{
	int rc=0;

  	
	if((major = register_chrdev(0, "unodio", &unodio_fops)) < 0)
	{
  		printk("register_chrdev failed!\n");
		return -ENODEV;
	}

	//find if dio pci device exists	
	//#if LINUX_VERSION_CODE > KERNEL_VERSION(2,6,0)
	//	dev = pci_get_device(ADVANTECH_VENDOR, UNO2679,NULL);
	//#else	
	//	dev = pci_find_device(ADVANTECH_VENDOR, UNO2679,NULL);	
	//#endif
	rc = pci_register_driver (&unodio_driver);
        if ( rc < 0 )
	{	
		return -ENODEV;
	}
	if(advdev)	//pci device exists
	{
		is_pci = 1;

	}
  	else	//for isa dio
  	{
		
		uno_function_init();
  		is_pci = 0;
  		dio_type = unodio_4channel;
  		//map bios address
		uc_ptaddr = ioremap_nocache( advhdbase, adv_range );
		if(strstr(uc_ptaddr,"2176") != NULL
		|| strstr(uc_ptaddr,"4672") != NULL)
		{
			dio_type = unodio_8channel;
		}
			
  		request_region( iobase, io_range, "unodio" );
		init_waitqueue_head( &adv_queue );
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 23)
		if ( request_irq (irqr, punodio_interrupt, IRQF_SHARED,
			"unodio", 
		&unodio_fops)) 
#else
		if ( request_irq (irqr, punodio_interrupt, SA_SHIRQ,
			"unodio", 
			&unodio_fops)) 
#endif
		{
			DEBUGPRINT ("IRQ %d is not free.\n", irqr);
			goto out_irq;
		}


		printk("=====================================================\n");
		printk("     DIO driver for UNO DIO  V%s [%s]\n", 
			ADVANTECH_DIO_VER, ADVANTECH_DIO_DATE);
		printk("     IO base address: 0x%XH\n", iobase);
		printk("     IRQ number:      %d\n", irqr );
		printk("     Advantech eAutomation Division.\n");
		printk("=====================================================\n");

		return 0;
	out:	
		release_region(iobase, 16);
		return 0;
 
	out_irq:
		free_irq(irqr, &unodio_fops);
		goto out;
  	}
	unodio_reset();	
  	return 0;
}
int (*pGetPowerStatus )( short* powerstatus );
int (*pDioReadPortByte) ( short port, short* value );
int (*pDioWritePortByte )(short port, short mask, short state );
int (*pDioReadBit) (short port, short bit, short* state );
int (*pDioWriteBit) (short port, short bit, short state );
int (*pDioGetCurrentDOByte) ( short port, short* value );
int (*pDioGetCurrentDOBit) (short port, short bit, short* state );
int (*pCounterClockConfig) ( short TimeBase );
int (*pCounterConfig )(short counter, short ClockPin, short GatePin, short OutPin );
int (*pCounterGateSet) ( short counter, short CTRGate );
int (*pCounterOutputMonitor) ( short counter, short* state );
int (*pCounterStartIO) ( short counter, short InitLoCount, short InitHiCount );
int (*pCounterTimerStart) ( short counter, long InitHWCount );
unsigned short (*pCounterReadIO) ( short counter );
int (*pCounterEventStart) ( short counter, short GateMode );
int (*pCounterEventRead32) (short* overflow, unsigned long* count );
int (*pCounterEventRead) ( short counter, short* overflow, unsigned long * count );
int (*pfreq_thread)( void * param );
int (*pCounterFreqStart) ( short counter, short GatePeriod,short GateMode );
int (*pCounterFreqRead) (short counter,unsigned long *timelapse,long *cntnum,long *kernelhz );
int (*pCounterPulseStart) (short counter, long  period, long  UpCycle, short GateMode )  ;
int (*pEventEnable) (short EventType, short Enabled, short Count );
int (*pCounterReset )( short counter );
int (*pEventConfig) ( short EventType, short RisingEdge );
int (*pLedEnable) ( short usMode );
int (*pLedToggle) ( unsigned short usMode,unsigned short usMask );
int (*pBuzzerEnable) ( short usMode );
int (*pWritePortByte) ( short offset, short ByteData );
int (*pReadPortByte) ( short offset, short* ByteData );
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,0)
void (*punodio_interrupt)(int    irq, void   *dev_id,struct pt_regs *regs )  ; 
#elif LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 19)
irqreturn_t (*punodio_interrupt)(int    irq, void   *dev_id,struct pt_regs *regs ) ;
#else
irqreturn_t (*punodio_interrupt)(int    irq, void   *dev_id ) ;    
#endif

module_init( unodio_init );
module_exit( unodio_cleanup );
 
MODULE_DESCRIPTION("Driver for ADVANTECH UNO DIO module");
MODULE_LICENSE("GPL");

//EXPORT_NO_SYMBOLS;
