// ************************************************************
// * Program        : counter.c 
// * Description    : Demo program for event counter function
// * Boards Supp.   : UNO DIO
// *
// * Revision       : 1.0.0.1
// * Date           : 01/09/2009          Advantech Co., Ltd.
// *************************************************************

#include "unodio.h"

int main ( void )
{	
	int fd, chan, ret;
	USHORT wOverflow;
	unsigned long dwReading;
	//int nKey;
    
	//Step 1: Display hardware and software settings for running this example
	printf("\n------ Event counter function testing ------\n");
  
	//Step 2: Open device
	fd = DeviceOpen();
  
	//Step 3: choose which channel do you want to use
	printf("\nPlease input parameters ---> counter channel:\n");
	scanf("%d", &chan);
  
	// Step 4: config clock pin, gate pin and out pin direction
	// ClockPin= CFG_CLOCK_INTERNAL or CFG_CLOCK_EXTERNAL
	// GatePin = CFG_GATE_INTERNAL  or CFG_GATE_EXTERNAL
	// OutPin  = CFG_OUTPIN_TO_DO   or CFG_OUTPIN_NON_DO
	ret = CounterConfig(chan, CFG_CLOCK_EXTERNAL, CFG_GATE_INTERNAL, CFG_OUTPIN_NON_DO);	
	if ( ret != 0 ) {
		return 0;
	}
  
	// Step 5: Start counter operation by CounterEventStart
	//----CTRGate  = CFG_CTR_START or CFG_CTR_STOP; // internal control to start or stop    
	ret = CounterEventStart(chan, CFG_CTR_START);
	if ( ret != 0 ) {
		return 0;
	}
  
	// Step 6: Read counter value by CounterEventRead in while loop
	//         and display counter value, exit when pressing any key
	int retval;
        fd_set rfds;
        struct timeval tv;
	char buf[128];

        while ( 1 )
        {
	        ret = CounterEventRead(chan, &wOverflow, &dwReading);
        	if (ret != 0) {
                	return 0;
	        }

	        printf("\nCounter value = %lu, press 'q' to exit.", dwReading);

                FD_ZERO(&rfds);
                FD_SET(0, &rfds);
                /* Wait up to one second */
                tv.tv_sec = 1;
                tv.tv_usec = 0;
	        retval = select( 1, &rfds, NULL, NULL, &tv);
        	/* Don't rely on the value of tv now! */
	        if (retval == -1)
        		perror("select()");
		else if ( retval )
         	{
            		/* FD_ISSET(0, &rfds) will be true. */
			if ( fgets( buf, sizeof(buf), stdin ) == NULL )
				perror("fgets()");
			else if ( buf[ 0 ] == 'q' )
			{
				printf("\n");
				break;
			}
	         }
	}

	// Step 7: Stop counter by CounterReset
	ret = CounterReset(chan);
	if (ret != 0) {
		return 0;
	}
  
	// Step 8: Close device
	 DeviceClose();
  
	return 0;
}
