// *************************************************************************
// * Program        	: diint.c
// * Description    	: Demo program for digital input with interrupt function
// * Boards Support.   	: UNO DIO
// *
// * Revision       : 1.0.0.1
// * Date           : 01/08/2009          Advantech Co., Ltd.
// *************************************************************************
#include <sys/time.h>
#include <sys/types.h>
#include <pthread.h>
#include "unodio.h"

int ret = 0;
int thread_exit = 0;
pthread_t thread_handle;
USHORT nEventType = 0;
static int int_num = 0;
//thread function to check events
void * check_event_func(void* p)
{
	unsigned short i_sta = 0;
	struct timeval tv,tv_new,tv_old;

	gettimeofday(&tv_old,NULL);
	while(!thread_exit)
	{
		tv.tv_sec = 1;
		tv.tv_usec = 0;
		i_sta = EventCheck(nEventType,tv);
		//printf("i_sta = 0x%x\n",i_sta);
		if( i_sta & EVT_DI0 )
		{
			gettimeofday(&tv_new,NULL);
			int_num ++;
			if((int_num %100)== 0)
				printf("DI0 interrupt occurred %.3f/s!\n",(float)int_num/(tv_new.tv_sec-tv_old.tv_sec));
			//printf("DI0 interrupt occurred!\n");
		}
		if( i_sta & EVT_DI1 )			
			printf("DI1 interrupt occurred!\n");
		if( i_sta == 0 )
			printf("timeout!\n");
	}
	return NULL;
}
int main ( void )
{
    	//Step 1: Display hardware and software settings
    	printf("------ DI interrupt function testing ------\n\n");

	//Step 2: Open device
    	ret = DeviceOpen();        
    	if ( ret == -1 )
	{
		printf("ads0 device open error!\n");
		return 1;
	}
    	
	// Step3: configure the DI interrupt for rising edge
    	nEventType = ADS_EVT_DI0 | ADS_EVT_DI1;	
    	//if (EventConfig(nEventType, EVENT_EDGE_RISING))
       	 //	return 1;

    	// Step4: enable and start the event function
    	nEventType = ADS_EVT_DI0 | ADS_EVT_DI1;
    	if (EventEnable(nEventType, 1, 1))
        	return 1;
	
	//Setp5: create a thread to check if events occur
	pthread_create(&thread_handle,NULL,check_event_func,NULL);
	printf("Press enter key to exit\n");
	getchar();
	thread_exit = 1;
	pthread_join(thread_handle,NULL);
	
	// Step6: disable and close the event function
	nEventType = ADS_EVT_DI0 | ADS_EVT_DI1;
	printf("Event disable!\n");
	if (EventEnable(nEventType, 0, 1))
        	return 1;

	// Step7: close device handle
	DeviceClose();	
	return 0;
}
