// ***********************************************************************
// * Program        : freq.c
// * Description    : Demo program for frequence measure function
// * Boards Supp.   : UNO DIO
// *
// * Revision       : 1.0
// * Date           : 01/08/2009          Advantech Co., Ltd.
// ***********************************************************************

#include "unodio.h"
#define SUCCESS 0

int main ( void )
{
	int  dwErrCde;
	int usChan;//, nKey;	
	float  fFreq = 0.0; 

	printf("\n------ Frequency measure function testing ------\n");

	//Step 1: Open device
	dwErrCde = DeviceOpen();
	if (dwErrCde == -1)
		return 1;

	// Step 2: input channel, only support 0 or 1
	printf("\nPlease input parameters ---> counter channel:\n");
	scanf("%d", &usChan);
	if ( ( usChan < 0 ) || ( usChan > 1 ) )
	{
		printf("Channel invalid!\n");
		return 1;
	}

	// Step 3: config clock pin, gate pin and out pin direction
	// ClockPin= CFG_CLOCK_INTERNAL or CFG_CLOCK_EXTERNAL
	// GatePin = CFG_GATE_INTERNAL  or CFG_GATE_EXTERNAL
	// OutPin  = CFG_OUTPIN_TO_DO   or CFG_OUTPIN_NON_DO
	dwErrCde = CounterConfig(usChan,/*CFG_CLOCK_INTERNAL*/CFG_CLOCK_EXTERNAL, CFG_GATE_INTERNAL, CFG_OUTPIN_TO_DO);	
	if ( dwErrCde != SUCCESS )
		return 1;

	//dwErrCde = CounterClockConfig(TIME_BASE_1K);
	//if ( dwErrCde != 0 )
	//	return 1;
		    
	// Step 4: Start frequency measurement operation by CounterFreqStart    
	//----CTRGate  = CFG_CTR_START or CFG_CTR_STOP; // internal control to start or stop
	printf("\nfrequency measurement starts ....(GatePeriod = 0, CTRGate = CFG_CTR_START) ");
	dwErrCde = CounterFreqStart(usChan, 0, CFG_CTR_START);
	if (dwErrCde != SUCCESS)
		return 1;
    
	sleep(5);  // Waiting sometime start to read freq's value

	// Step 5: Read counter value by CounterFreqRead in while loop
	//         and display counter value, exit when pressing any key
	
        int retval;
        fd_set rfds;
        struct timeval tv;
        char buf[128];

        while ( 1 )
        {
                dwErrCde = CounterFreqRead( usChan, &fFreq );
                if ( dwErrCde != 0) {
                        return 0;
                }

                printf("\nFrequency = %fHz, press 'q' to exit.", fFreq);

                FD_ZERO( &rfds );
                FD_SET( 0, &rfds );
                // Wait up to one second 
                tv.tv_sec = 5;
                tv.tv_usec = 0;
                retval = select( 1, &rfds, NULL, NULL, &tv);
                // Don't rely on the value of tv now! 
                if (retval == -1)
                        perror("select()");
                else if ( retval )
                {
                        // FD_ISSET(0, &rfds) will be true. 
                        if ( fgets( buf, sizeof(buf), stdin ) == NULL )
                                perror("fgets()");
                        else if ( buf[ 0 ] == 'q' )
                        {
                                printf("\n");
                                break;
                        }
                 }
        }

	// Step 6: Stop counter by CounterReset
	dwErrCde = CounterReset( usChan );
	if ( dwErrCde != SUCCESS )	
	{
		printf("CounterReset error!\n");
		return 1;
	}

	// Step 7: Close device
	DeviceClose();

	return 0;		
}
