// ***********************************************************************
// * Program        : timer.c
// * Description    : Demo program for timer function with interrupt
// * Boards Supp.   : UNO DIO
// *
// * Revision       : 1.0.0.1
// * Date           : 01/08/2009          Advantech Co., Ltd.
// ***********************************************************************
#include <pthread.h>
#include "unodio.h"
#define SUCCESS 0

int thread_exit = 0;
pthread_t thread_handle;
USHORT nEventType = 0;
USHORT usChan;

//thread function to check events
void * check_event_func(void* p)
{
	ULONG  dwReading = 0;
	USHORT   wOverflow = 0 ;
	unsigned short i_sta = 0;
	struct timeval tv;
	int ret = 0;
	
	while(!thread_exit)
	{
		tv.tv_sec = 1;
		tv.tv_usec = 0;
		
		ret = CounterEventRead(usChan, &wOverflow, &dwReading);
       		if (ret != 0)
		{
			printf("Read counter failed\n");
	       		return NULL;
		}
       		printf("Counter value = %lu\n", dwReading);  

       	
		i_sta = EventCheck(nEventType,tv);
		//printf("eventtype=%x",i_sta);
		if ( i_sta & EVT_CNT0 )
			printf("Counter0 interrupt occurred!\n");
		if ( i_sta & EVT_CNT1 )
			printf("Counter1 interrupt occurred!\n");
		if( i_sta == 0)
			printf("timeout!\n");
	}
	printf("event check thread exit.\n");
	return NULL;
}

int main ( void )
{
	int dwErrCde, fd;
	ULONG  ulHardwareCounter;
	int nTimeBase, nTimeBaseHW = TIME_BASE_100;
 	
	//Step 1: Display 
	printf("Timer interrupt with interrupt function testing....\n\n");

	//Step 2: Open device
	fd = DeviceOpen();
	if ( fd == -1 )
	{
		printf("Device Open failed!\n");
		return 1;
	}
    
	//Step 3: choose which channel do you want to use and reset it
	printf("\nPlease input parameters ---> counter channel:");
	scanf("%d", (int*)&usChan);
	CounterReset(usChan);

	switch ( usChan )
	{
	case COUNTER_0:
		nEventType = ADS_EVT_CONT0;
	break;
	case COUNTER_1:
		nEventType = ADS_EVT_CONT1;
	break;
	case COUNTER_32:
		nEventType = ADS_EVT_CONT0 | ADS_EVT_CONT1;
	break;
	default:
		printf( "Counter channel is invalid!\n" );
		return 1;
	}
	// Step 4: Config counter (items: time base chosing)
	printf("\nPlease input the time base mode: ");
	printf("\n0 -- TIME BASE is 100K ");
	printf("\n1 -- TIME BASE is 10K  ");
	printf("\n2 -- TIME BASE is 1K   ");
	printf("\n3 -- TIME BASE is 100  \n");
	scanf("%d", &nTimeBase); // make sure the input is from 0 to 3
	if (nTimeBase == 0) nTimeBaseHW = TIME_BASE_100K;
	if (nTimeBase == 1) nTimeBaseHW = TIME_BASE_10K;
	if (nTimeBase == 2) nTimeBaseHW = TIME_BASE_1K;
	if (nTimeBase == 3) nTimeBaseHW = TIME_BASE_100;

	dwErrCde = CounterClockConfig(nTimeBaseHW);	
	if (dwErrCde != 0 )
		return 1;

	// Step 5: enable and start the event function
	if (EventEnable(nEventType, 1, 1))
	        return 1;

	// Step 6: Start counter operation by CounterEventStart
	ulHardwareCounter = 0xffff;	// for internal clock = 100K
	dwErrCde = CounterTimerStart(usChan, ulHardwareCounter);
	if (dwErrCde != SUCCESS)
		return 1;

        // Create a thread to check event
    	pthread_create(&thread_handle,NULL,check_event_func,NULL);
	getchar();
	printf("Press enter key to exit\n");
	getchar();
	thread_exit = 1;
	pthread_join(thread_handle,NULL);                                  
	
    	// Step 7: disable and close the event function
	if (EventEnable(nEventType, 0, 1))
        	return (1);

	// Step 8: Stop counter by CounterReset
	dwErrCde = CounterReset(usChan);
	if (dwErrCde != SUCCESS)
		return (1);

	// Step 9: Close device
	dwErrCde = DeviceClose();

	return 0;
}
