#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/time.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <string.h>
#include <signal.h>
#include "unodio_ioctl.h"

typedef unsigned short USHORT;
typedef unsigned long ULONG;

// Counter channel number
#define COUNTER_0     0x0  // 16-bit counter 0
#define COUNTER_1     0x1  // 16-bit counter 1
#define COUNTER_32    0x2  // Cascaded 32-bit counter of counter 0 and counter 1

// interrupt flag & clear (offset 7)
#define INT_DI0_MASK             0x1
#define INT_DI1_MASK             0x2
#define INT_CNTA_MASK            0x4
#define INT_CNTB_MASK            0x8

// Buzzer Mode
#define BEEP_DISABLE  0x0 // disable alarm buzzer
#define BEEP_ON       0x1 // beep on
#define BEEP_SHORT    0x3 // short beep
#define BEEP_MIDDLE   0x5 // normal beep
#define BEEP_LONG     0x7 // long beep
 
// LED Mode
#define LED_DISABLE  0x0  // disable diagnostic LED
#define LED_ON       0x1  // Light on
#define LED_FAST     0x3  // fast flicker
#define LED_NORMAL   0x5  // normal flick
#define LED_SLOW     0x7  // slow flicker

// Event Type
#define ADS_EVT_DI0     0x0001  // for DI 0
#define ADS_EVT_DI1     0x0002  // for DI 1
#define ADS_EVT_DI2     0x0004  // for DI 2
#define ADS_EVT_DI3     0x0008  // for DI 3
#define ADS_EVT_DI4     0x0010  // for DI 4
#define ADS_EVT_DI5     0x0020  // for DI 5
#define ADS_EVT_DI6     0x0040  // for DI 6
#define ADS_EVT_DI7     0x0080  // for DI 7
#define ADS_EVT_DI8     0x0100  // for DI 8
#define ADS_EVT_CONT0   0x0004  // for counter 0
#define ADS_EVT_CONT1   0x0008  // for counter 1 or cascaded 32-bit counter

#define EVENT_EDGE_RISING  1  // interrpt trigger edge for rising edge
#define EVENT_EDGE_FALLING 0  // interrpt trigger edge for falling edge
/*
#define EVENT_DI0	40
#define EVENT_DI1	41
#define EVENT_CNT0	42
#define EVENT_CNT1     43
*/
// Internal clock's time base selecting (TimerBase)
#define TIME_BASE_100K  0x00    // time base 100K Hzpci_board_num_t
#define TIME_BASE_10K   0x10    // time base 10K Hz
#define TIME_BASE_1K    0x20    // time base 1K Hz
#define TIME_BASE_100   0x30    // time base 100 Hz

// clock source selecting (for ClockPin parameter)
#define CFG_CLOCK_INTERNAL  0   // clock source from internal clock
#define CFG_CLOCK_EXTERNAL  1   // clock source from external clock
                                // Counter 0 from DI1
                                // Counter 1 and cascaded 32-bit counter from DI3

// Gate tigger selecting (for GatePin paramete)
#define CFG_GATE_INTERNAL    0  // gate input from internal trigger
#define CFG_GATE_EXTERNAL    1  // gate input from external trigger
                                // Counter 0 from DI0
                                // Counter 1 and cascaded 32-bit counter from DI2

// Output pin selecting (for OutPin parameter)
#define CFG_OUTPIN_NON_DO    0  // Counter not output to DO channel
#define CFG_OUTPIN_TO_DO     1  // Counter output to DO channel
                                // Counter 0 output to DO2
                                // Counter 1 and cascaded 32-bit counter to DO3

// In usGATEPin=CFG_GATE_INTERNAL for CTRGATE parameter's definition
#define CFG_CTR_STOP    0       // counter pause
#define CFG_CTR_START   1       // counter start or resume
                                // Counter output - Output pin selecting (for OutPin parameter)
#define CFG_OUTPIN_NON_MONITOR  0  // Counter not output to CTR0Out or CTR1Out
#define CFG_OUTPIN_MONITOR      1  // Counter output pin to CTR0Out or CTR1Out

#define POWER1_OK	0x01
#define POWER2_OK	0x02
#define POWER_OK	0x04

extern int fd;

int   DeviceOpen ( void );
int   DeviceClose ( void );
int   GetIOChannelNum( USHORT Port, USHORT DIOType);//DIOType (0:DI, 1:DO)
int   GetPowerStatus ( USHORT * usState );
ULONG DIReadPortByte(USHORT port, USHORT *value);
ULONG DOWritePortByte(USHORT port, USHORT mask, USHORT value);
ULONG DIReadBit(USHORT port, USHORT bit, USHORT *state);
ULONG DOWriteBit(USHORT port, USHORT bit, USHORT state);
ULONG DOGetCurrentByte(USHORT port, USHORT *value);
ULONG DOGetCurrentBit(USHORT port, USHORT bit, USHORT *state);
ULONG CounterTimerStart(USHORT counter, int InitHWCount);
ULONG CounterEventStart(USHORT counter, USHORT usCTRGate);
ULONG CounterEventRead(USHORT counter, USHORT *overflow, ULONG *count);
ULONG CounterFreqStart(USHORT counter, USHORT GatePeriod, USHORT usCTRGate);
ULONG CounterFreqRead(USHORT counter, float *freq);
ULONG CounterPulseStart(USHORT counter, float period, float UpCycle, USHORT usCTRGate);
ULONG CounterReset(USHORT counter);
ULONG CounterClockConfig(USHORT TimeBase);
ULONG CounterConfig(USHORT counter, USHORT ClockPin, USHORT GatePin, USHORT OutPin);
ULONG CounterGateSet(USHORT counter, USHORT CTRGate);
ULONG CounterOutputMonitor(USHORT counter, USHORT *state);
ULONG WritePortByte(USHORT offset, USHORT ByteData);
ULONG ReadPortByte(USHORT offset, USHORT* ByteData);
ULONG EventConfig(USHORT EventType, USHORT RisingEdge);
ULONG EventEnable(USHORT EventType, USHORT Enabled, USHORT Count);
USHORT EventCheck(USHORT EventType,struct timeval timeout);
ULONG LedEnable(USHORT Mode);
ULONG LedToggle(USHORT Mode,USHORT Mask);
ULONG BuzzerEnable(USHORT Mode);
ULONG ProgramLedRead( USHORT *usData );
ULONG ProgramLedWrite( USHORT usData );
ULONG ProgramLedModeEnable( USHORT ledChannel, USHORT Enabled );
ULONG ProgramLedModeConfig( USHORT usMode );

#define EVT_DI0    0x0001
#define EVT_DI1    0x0002
#define EVT_CNT0   0x0004
#define EVT_CNT1   0x0008

#define GETEVTSTA_DI0           0x0001
#define GETEVTSTA_DI1           0x0002
#define GETEVTSTA_CNT0          0x0004
#define GETEVTSTA_CNT1          0x0008
 
