/* #############################################################################
   *****************************************************************************                  
               Copyright (c) 2005, Advantech eAutomation Division
                              Advantech Co., Ltd.
                               Linux DIO driver
        THIS IS AN UNPUBLISHED WORK CONTAINING CONFIDENTIAL AND PROPRIETARY
                 INFORMATION WHICH IS THE PROPERTY OF ADVANTECH AUTOMATION CORP.
      ANY DISCLOSURE, USE, OR REPRODUCTION, WITHOUT WRITTEN AUTHORIZATION FROM
                 ADVANTECH AUTOMATION CORP., IS STRICTLY PROHIBITED.
   *****************************************************************************
   #############################################################################
   File:        libunodio.c
   Change log:  Version 1.00 <2007/04/28>
   Description:	UNO DIO driver lib for user application

   Permission to use, copy, modify, and distribute this software and its
   documentation for any purpose and without fee is hereby granted, provided
   that the above copyright notice appear in all copies and that both that
   copyright notice and this permission notice appear in supporting
   documentation.  This software is provided "as is" without express or
   implied warranty.
  ----------------------------------------------------------------------------*/

#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <fcntl.h>
#include <stdio.h>
#include <sys/ioctl.h>
#include <string.h>
#include <signal.h>
#include <time.h>
#include "../include/unodio.h"
#include "../include/unodio_ioctl.h"

int fd = 0;
int unodio_type = 0;
#define	PLED_OFFSET              0x14
#define LED_OFFSET               0x10
#define LED_OFFSET1              0x14
enum unodio_type {
	unodio_4channel = 0, // for UNOs ISA DIO, 4 DIO channel device
	unodio_8channel,     // for UNO-2052(E),UNO_4672,2176, 8 DIO channel
	unodio_2679,         // for UNO-2679, 2 port, 8DIO for port0 and 1 DI for port1
	unodio_1171,
} ;   
int DeviceOpen ( void )
{
	if((fd = open("/dev/unodio", O_RDWR)) < 0) {
      		printf("cannot open device file /dev/unodio!\n");
      		return -1;
    	}
	GetDIOType();	
    	return fd;
}

int DeviceClose ( void )
{
	if(close(fd)) {
		printf("close device file error!\n");
		return -1;
	}
		
	return 0;
}

ULONG DIReadPortByte(USHORT port, USHORT *value)
{
	ULONG status;
	USHORT data[2];
	if (port > 0 && unodio_type != unodio_2679
		|| port > 1)
	{
		printf("Invalid Port!\n");
		return -1;
	}
	data[0] = port;
	data[1] = *value;
	status = ioctl(fd, DIO_READ_PORT_BYTE, data);
	if(status) {
		printf("DioReadPortByte error!\n");
		return status;
	}
	*value = data[1];
	return status;
}

ULONG DOWritePortByte(USHORT port, USHORT mask, USHORT value)
{
	ULONG status;
	USHORT data[2];
	if (port > 0)
	{
		printf("Invalid Port!\n");
		return -1;
	}
	data[0] = mask;
	data[1] = value;
	
	status = ioctl(fd, DIO_WRITE_PORT_BYTE, data);
	
	if(status) {
		printf("DioWritePortByte error!\n");
		return status;
	}
	
	return status;
}

ULONG DIReadBit(USHORT port, USHORT bit, USHORT *state)
{
	ULONG status;
	USHORT data[2];
	if (port > 0 && unodio_type != unodio_2679
		|| port > 1)
	{
		printf("Invalid Port!\n");
		return -1;
	}
	else if (bit > 3 && unodio_type == unodio_4channel)
	{
		printf("Invalid Bit!\n");
		return -1;
	}
	data[0] = bit;
	
	status = ioctl(fd, DIO_READ_BIT, data);
	
	if(status) {
		printf("DioReadBit error!\n");
		return status;
	}
	
	*state = data[1];
	
	return status;
}

ULONG DOWriteBit(USHORT port, USHORT bit, USHORT state)
{
	ULONG status;
	USHORT data[2];
	if (port > 0)
	{
		printf("Invalid Port!\n");
		return -1;
	}
	else if (unodio_type == unodio_4channel && bit > 3)
	{
		printf("Invalid Bit!\n");
		return -1;
	}
	data[0] = bit;
	data[1] = state;
	
	status = ioctl(fd, DIO_WRITE_BIT, data);
	
	if(status) {
		printf("DioWriteBit error!\n");
		return status;
	}
	
	return status;
}

ULONG DOGetCurrentByte(USHORT port, USHORT *value)
{
	ULONG status;
	if (port > 0)
	{
		printf("Invalid Port!\n");
		return -1;
	}
	status = ioctl(fd, DIO_GET_DO_BYTE, value);
	if(status) {
		printf("DioGetCurrentDOByte error!\n");
		return status;
	}
	
	return status;
}

ULONG DOGetCurrentBit(USHORT port, USHORT bit, USHORT *state)
{
	ULONG status;
	USHORT data[2];
	if (port > 0)
	{
		printf("Invalid Port!\n");
		return -1;
	}
	else if (unodio_type == unodio_4channel && bit > 4)
	{
		printf("Invalid Bit!\n");
		return -1;
	}
	
	data[0] = bit;
	
	status = ioctl(fd, DIO_GET_DO_BIT, data);
	
	if(status) {
		printf("DioGetCurrentDoBit error!\n");
		return status;
	}
	
	*state = data[1];
	
	return status;
}	

ULONG CounterTimerStart(USHORT counter, int InitHWCount)
{
	ULONG status;
	ULONG data[2];
	if (unodio_type == unodio_2679)
	{
		printf("This function is not support for the device!\n");
		return -1;
	}
	data[0] = (ULONG)counter;
	data[1] = InitHWCount;
	
	status = ioctl(fd, COUNTER_TIMER_START, data);
	if(status) {
		printf("CounterTimerStart error!\n");
		return status;
	}
	
	return status;
}

ULONG CounterEventStart(USHORT counter, USHORT usCTRGate)
{
	ULONG status;
	USHORT data[2];
	if (unodio_type == unodio_2679)
	{
		printf("This function is not support for the device!\n");
		return -1;
	}
	data[0] = counter;
	data[1] = usCTRGate;
	
	status = ioctl(fd, COUNTER_EVENT_START, data);
	if(status) {
		printf("CounterEventStart error!\n");
		return status;
	}
	
	return status;
}


ULONG CounterEventRead(USHORT counter, USHORT *overflow, ULONG *count)
{
	ULONG status;
	ULONG data[3];
	if (unodio_type == unodio_2679)
	{
		printf("This function is not support for the device!\n");
		return -1;
	}
	data[0] = (ULONG)counter;
	
	status = ioctl(fd, COUNTER_EVENT_READ, data);
	if(status) {
		printf("CounterEventStart error!\n");
		return status;
	}

	*overflow = (short)data[1];
        *count = data[2];
	
	return status;
}

ULONG CounterFreqStart(USHORT counter, USHORT GatePeriod, USHORT usCTRGate)
{
	ULONG status;
	USHORT data[3];
	if (unodio_type == unodio_2679)
	{
		printf("This function is not support for the device!\n");
		return -1;
	}
	data[0] = counter;
	data[1] = GatePeriod;
	data[2] = usCTRGate;
	
	status = ioctl(fd, COUNTER_FREQ_START, data);
	if(status) {
		printf("CounterFreqStart error!\n");
		return status;
	}
	
	return status;
}

// Joshua modified 02/28/2005
ULONG CounterFreqRead(USHORT counter, float *freq)
{
	ULONG status;
	ULONG data[4];	
	if (unodio_type == unodio_2679)
	{
		printf("This function is not support for the device!\n");
		return -1;
	}
	data[0] = counter;
	
	status = ioctl(fd, COUNTER_FREQ_READ, data);
	if(status) {
		printf("CounterFreqRead error!\n");
		return status;
	}
	*freq = (float)data[2] * (float)data[3] / (float)data[1];
	
	return status;
}
// modified end

// Joshua modified 02/28/2005 
ULONG CounterPulseStart ( 
	USHORT counter, 
	float period, 
	float UpCycle, 
	USHORT usCTRGate )
{
	ULONG status;
	ULONG data[4];
	if (unodio_type == unodio_2679)
	{
		printf("This function is not support for the device!\n");
		return -1;
	}
	data[0] = (ULONG)counter;
	data[1] = (ULONG)(period * 100000);  // convert to 100k timebase
	data[2] = (ULONG)(UpCycle);
	data[3] = (ULONG)usCTRGate;
	status = ioctl(fd, COUNTER_PULSE_START, data);
	if(status) {
		printf("CounterPulseStart error!\n");
		return status;
	}
	
	return status;
}
// modified end

ULONG CounterReset(USHORT counter)
{
	ULONG status;
	USHORT data[1];
	if (unodio_type == unodio_2679)
	{
		printf("This function is not support for the device!\n");
		return -1;
	}
	data[0] = counter;
	
	status = ioctl(fd, COUNTER_RESET, data);
	if(status) {
		printf("CounterReset error!\n");
		return status;
	}
	
	return status;
}

ULONG CounterClockConfig(USHORT TimeBase)
{
	ULONG status;
	USHORT data[1];
	if (unodio_type == unodio_2679)
	{
		printf("This function is not support for the device!\n");
		return -1;
	}
	data[0] = TimeBase;
	
	status = ioctl(fd, COUNTER_CLOCK_CFG, data);
	if(status) {
		printf("CounterClockConfig error!\n");
		return status;
	}
	
	return status;
}

ULONG CounterConfig(USHORT counter, USHORT ClockPin, USHORT GatePin, USHORT OutPin)
{
	
	ULONG status;
	USHORT data[4];
	if (unodio_type == unodio_2679)
	{
		printf("This function is not support for the device!\n");
		return -1;
	}
	data[0] = counter;
	data[1] = ClockPin;
	data[2] = GatePin;
	data[3] = OutPin;
	
	status = ioctl(fd, COUNTER_CFG, data);
	if(status) {
		printf("CounterConfig error!\n");
		return status;
	}
	
	return status;	
}

ULONG CounterGateSet(USHORT counter, USHORT CTRGate)
{
	if (unodio_type == unodio_2679)
	{
		printf("This function is not support for the device!\n");
		return -1;
	}
	ULONG status;
	USHORT data[2];
	
	data[0] = counter;
	data[1] = CTRGate;
	
	status = ioctl(fd, COUNTER_GATE_SET, data);
	if(status) {
		printf("CounterGateSet error!\n");
		return status;
	}
	
	return status;
}


ULONG CounterOutputMonitor(USHORT counter, USHORT *state)
{
	if (unodio_type == unodio_2679)
	{
		printf("This function is not support for the device!\n");
		return -1;
	}
	ULONG status;
	float data[2];
	
	data[0] = (float)counter;
	
	status = ioctl(fd, COUNTER_FREQ_READ, data);
	if(status) {
		printf("CounterFreqRead error!\n");
		return status;
	}
	
	*state = data[1];
	
	return status;		
}

ULONG WritePortByte(USHORT offset, USHORT ByteData)
{
	ULONG status;
	USHORT data[2];
	
	data[0] = offset;
	data[1] = ByteData;
	
	status = ioctl(fd, WRITE_PORT_BYTE, data);
	if(status) {
		printf("WritePortByte error!\n");
		return status;
	}
	
	return status;	
}

ULONG ReadPortByte(USHORT offset, USHORT* ByteData)
{
	ULONG status;
	USHORT data[2];
	
	data[0] = offset;
	
	status = ioctl(fd, READ_PORT_BYTE, data);
	if(status) {
		printf("ReadPortByte error!\n");
		return status;
	}
	
	*ByteData = data[1];
	
	return status;	
}

ULONG EventConfig(USHORT EventType, USHORT RisingEdge)
{
	ULONG status;
	USHORT data[2];
	if (unodio_type == unodio_2679)
	{
		printf("This function is not support for the device!\n");
		return -1;
	}
	data[0] = EventType;
	data[1] = RisingEdge;
	
	status = ioctl(fd, EVENT_CFG, data);
	if(status) {
		printf("EventConfig error!\n");
		return status;
	}
	
	return status;		
}

ULONG EventEnable(USHORT EventType, USHORT Enabled, USHORT Count)
{
	ULONG status;
	USHORT data[3];
	
	data[0] = EventType;
	data[1] = Enabled;
	data[2] = Count;
	
	status = ioctl(fd, EVENT_ENABLE, data);
	if(status) {
		printf("EventEnable error!\n");
		return status;
	}
	
	return status;		
}
USHORT EventCheck(USHORT EventType,struct timeval timeout)
{	
	int retval;
	struct timeval tv_begin, tv_end, tv_elapsed;
	struct timeval * tv_timeout;
	USHORT i_sta = 0;
	fd_set rfds;
  	
	FD_ZERO(&rfds);
	FD_SET(fd, &rfds);
	/*
	if(EventType & ADS_EVT_DI0)
		i_sta |= GETEVTSTA_DI0;
	if(EventType & ADS_EVT_DI1)
		i_sta |= GETEVTSTA_DI1;
	if(EventType & ADS_EVT_DI2)
		i_sta |= GETEVTSTA_DI2;
	if(EventType & ADS_EVT_DI3)
		i_sta |= GETEVTSTA_DI3;
	
	if(EventType & ADS_EVT_CONT0)
		i_sta |= GETEVTSTA_CNT0;
	if(EventType & ADS_EVT_CONT1)
		i_sta |= GETEVTSTA_CNT1;
	EventType = i_sta;
	*/
	tv_timeout = &timeout;
	if(tv_timeout->tv_sec<0 || tv_timeout->tv_usec<0)
		tv_timeout = NULL;
	gettimeofday(&tv_begin,NULL);
	while(1)
	{
		retval = select(FD_SETSIZE, NULL, NULL, &rfds, tv_timeout);
		if(retval > 0)
		{
			if(FD_ISSET(fd, &rfds))
                	{	
				//i_sta = EventType;
				retval = ioctl ( fd, GETEVENT, &i_sta );
				if ( i_sta & EventType ||((timeout.tv_sec==0) && (timeout.tv_usec==0)))
					return i_sta;
				else
				{
					if((timeout.tv_sec<0)||timeout.tv_usec<0)
						continue;
					gettimeofday(&tv_end,NULL);
					timersub(&tv_end,&tv_begin,&tv_elapsed);
					if(tv_elapsed.tv_sec > timeout.tv_sec)
					{
						return 0;
					}
					else if((tv_elapsed.tv_sec == timeout.tv_sec)&&(tv_elapsed.tv_usec > timeout.tv_usec))
					{
						return 0;
					}
					timersub(tv_timeout,&tv_elapsed,tv_timeout);
					continue;
				}
			}
		}
		if(retval == 0)	//time out
			return 0;	
	} //end of while
	return 0;
}
ULONG LedToggle(USHORT Mode,USHORT Mask)
{
	ULONG status;
	USHORT data[2];
	data[0] = Mode;
	data[1] = Mask;
	status = ioctl(fd,LED_TOGGLE,data);
	if (status)
	{
		printf("LEDToggle error\n");
		return status;
	}
	return status;
	
}

// *****************************************************************************
// Design Notes:  
// -----------------------------------------------------------------------------
ULONG ProgramLedRead( USHORT *usData  )
{
	
	return ReadPortByte(PLED_OFFSET, usData);
}

// *****************************************************************************
// Design Notes:  
// -----------------------------------------------------------------------------
ULONG ProgramLedWrite( USHORT usData )
{
	USHORT tmpLedValue;
	if( unodio_type == unodio_1171 ) 
	{
		ReadPortByte( LED_OFFSET, &tmpLedValue);
	
		WritePortByte( LED_OFFSET, tmpLedValue | 0x1 );
	}
	return WritePortByte(PLED_OFFSET, usData)  ;  
}

// *****************************************************************************
// Design Notes:  
// -----------------------------------------------------------------------------
ULONG ProgramLedModeEnable( USHORT ledChannel, USHORT Enabled )
{

	USHORT  usReadBack;
	USHORT  usData;
	
  	if( unodio_type != unodio_1171 ) 
	{
		printf("This function is not support for the device\n");
		return -1;
	}	

	ReadPortByte( PLED_OFFSET+1 , &usReadBack);

	if (Enabled == 1)
	{
		usData = ( 1 << ledChannel ) | usReadBack ;
	}
	else
	{
		usData = ~( 1 << ledChannel ) & usReadBack;
	}
	WritePortByte( PLED_OFFSET+1, usData & 0xF );
	
	return 0;       
}

// *****************************************************************************
// Design Notes:  
// -----------------------------------------------------------------------------
ULONG ProgramLedModeConfig( USHORT usMode )
{
  if( unodio_type != unodio_1171 ) 
  {
	printf("This function is not support for the device\n");
	return -1;
   }
   // Mode Description
   // LED_FAST     0x3  // fast flicker
   // LED_NORMAL   0x5  // normal flick
   // LED_SLOW     0x7  // slow flicker    

   WritePortByte( LED_OFFSET, usMode | 0x1 );

   return 0;       
}

ULONG LedEnable(USHORT Mode)
{
	if (unodio_type == unodio_2679)
	{
		printf("This function is not support for the device\n");
		return -1;
	}
	ULONG status;
	USHORT data[1];
	
	data[0] = Mode;
	
	status = ioctl(fd, LED_ENABLE, data);
	if(status) {
		printf("LedEnable error!\n");
		return status;
	}
	
	return status;	
}

ULONG BuzzerEnable(USHORT Mode)
{
	if (unodio_type == unodio_2679)
	{
		printf("This function is not support for the device ");
		return -1;
	}
	ULONG status;
	USHORT data[1];
	
	data[0] = Mode;
	
	status = ioctl(fd, BUZZER_ENABLE, data);
	if(status) {
		printf("BuzzerEnable error!\n");
		return status;
	}
	
	return status;	
}
int GetIOChannelNum( USHORT Port, USHORT DIOType)
{
        switch (unodio_type)
	{
	case unodio_4channel:
		if (Port == 0)
			return 4;
		else 
			return 0;     
	
	case unodio_8channel:
	case unodio_1171:
		if (Port == 0)
			return 8;
		else 
			return 0;
	case unodio_2679:
		if ( Port == 0)
			return 8;
		else if ( Port == 1 && DIOType == 0 )
			return 1;
		else 
			return 0;
	default:
		return 0;
	}
}

int GetDIOType()
{
	ULONG status;
	int DIOType;

	status = ioctl ( fd, DIO_TYPE, &DIOType);
	if( status )
	{
		printf( "Get IO channel number error!\n" );
		return status;
	}
	unodio_type = DIOType;
	return 0;
}

int GetPowerStatus ( USHORT * usState )
{
	ULONG status;
	status = ioctl( fd, GETPOWER, usState );
	//ioctl return -1 indicates not supporting geting power status
	return status;
}
